var AlignDialog = new Object();


/*----------------------------------------------------------------------
 *  Item
 *----------------------------------------------------------------------
 */
 
function Item(domNodes) {
    this.children = mapNew(AbstractDomNode, domNodes);
}

Item.prototype.createTemplate = function() {
    return new Template(
        map(
            function(adnNode) {
                return TemplateNode.createTemplateNode(adnNode);
            },
            this.children
        )
    );
}

/*----------------------------------------------------------------------
 *  AlignDialog
 *----------------------------------------------------------------------
 */

AlignDialog.init = function() {
    AlignDialog._data = window.arguments[0];
    AlignDialog._nodes = AlignDialog._data.nodes;//.slice(0,5);
    AlignDialog._items = mapNew(Item, split(AlignDialog._nodes, 1));
    
    //inspect(AlignDialog._itemNodes);
    
    if (AlignDialog._items.length > 0) {
        try {
            var template = Template.createTemplateFromItems(AlignDialog._items);
            
            for (var i = 0; i < AlignDialog._items.length; i++) {
                template.bind(AlignDialog._items[i]);
            }
            
            /*
             *  Print out a table of the bindings
             */
            var headers = [ 
                { label: "Template", id: "template" }, 
                { label: "Field", id: "field" },
                { label: "Hits", id: "hits" },
                { label: "Misses", id: "misses" }
            ];
            for (var i = 0; i < AlignDialog._items.length; i++) {
                headers.push({ label: "Item " + (i + 1), id: "" + i });
            }
            
            var dataNodes = [];
            for (var i = 0; i < template.children.length; i++) {
                var adns = map(function(item) { return item.children[i]; }, AlignDialog._items);
                dataNodes.push(new DataNode(template.children[i], adns, null));
            }
            
            var richtree = new RichTree(headers, dataNodes);
            document.getElementById("content-div").appendChild(richtree.element);
            
            AlignDialog._template = template;
        } catch (e) {
            alert(e);
        }
    }
}

function DataNode(templateNode, adns, parentDataNode) {
    this.parent = parentDataNode;
    this._templateNode = templateNode;
    this._adns = adns;
}

DataNode.prototype = {
    _XHTMLNS: "http://www.w3.org/1999/xhtml",
    constructCell: function(cell, columnId, opened) {
        if (this.parent) {
            this._constructCell(cell, columnId, opened);
        } else {
            this._constructFragmentCell(cell, columnId, opened);
        }
    },
    _constructCell: function(cell, columnId, opened) {
        var div = document.createElementNS(this._XHTMLNS, "html:div");
        var className = "padded";
        
        if (columnId == "template") {
            div.appendChild(document.createTextNode(this._templateNode.sampleADN.nodeName));
        } else if (columnId == "field") {
            div.appendChild(document.createTextNode(
                (this._templateNode.fieldName) ? this._templateNode.fieldName : ""));
        } else if (columnId == "hits") {
            div.appendChild(document.createTextNode(this._templateNode.hits));
        } else if (columnId == "misses") {
            div.appendChild(document.createTextNode(this._templateNode.misses));
        } else {
            var index = parseInt(columnId);
            var adn = this._adns[index];
            if (adn) {
                if (!opened || this.children.length == 0 || adn.nodeName == "IMG") {
                    if (adn.nodeName == "IMG") {
                        var img = document.createElementNS(this._XHTMLNS, "html:img");
                        img.src = adn.images[0];
                        
                        div.appendChild(img);
                    } else {
                        var text = adn.text;
                        div.appendChild(document.createTextNode(
                            text.length > 100 ? (text.substr(0, 100) + "...") : text));
                        div.title = text;
                            
                        var images = adn.images;
                        for (var i = 0; i < images.length; i++) {
                            var img = document.createElementNS(this._XHTMLNS, "html:img");
                            img.src = images[i];
                            
                            div.appendChild(img);
                        }
                        
                    }
                    className += " limited-width";
                }
            } else {
                className += " missing";
                div.appendChild(document.createTextNode("missing"));
            }
        }
        div.className = className;
        
        cell.appendChild(div);
    },
    _constructFragmentCell: function(cell, columnId, opened) {
        if (columnId == "template") {
            var div = document.createElementNS(this._XHTMLNS, "html:div");
            div.className = "padded";
            
            div.appendChild(document.createTextNode("fragment"));
            cell.appendChild(div);
        }
    },
    get hasChildren() {
        return true;
    },
    get children() {
        if (!this._children) {
            this._expand();
        }
        return this._children;
    },
    _expand : function() {
        var templateNode = this._templateNode;
        var adns = this._adns;
        
        var childNodes = [];
        
        var childTemplateNodes = templateNode.children;
        var adnIndices = map(function(adn) { return 0; }, adns);
        for (var i = 0; i < childTemplateNodes.length; i++) {
            var childTemplateNode = childTemplateNodes[i];
            var childADNS = [];
            
            for (var j = 0; j < adns.length; j++) {
                var childADN = null;
                
                if (adns[j]) {
                    if (adnIndices[j] < adns[j].children.length) {
                        var candidateChildADN = adns[j].children[adnIndices[j]];
                        if (candidateChildADN.binding && 
                            candidateChildADN.binding.templateNodeIndex == i) {
                            
                            childADN = candidateChildADN;
                            adnIndices[j]++;
                        }
                    }
                }
                
                childADNS.push(childADN);
            }
            
            if (childTemplateNode.sampleADN.nodeType == 1) {
                childNodes.push(new DataNode(childTemplateNode, childADNS, this));
            } else {
                childNodes.push(new TextDataNode(childTemplateNode, childADNS, this));
            }
        }
        
        this._children = childNodes;
    }
}


function TextDataNode(templateNode, adns, parentDataNode) {
    this.parent = parentDataNode;
    this._templateNode = templateNode;
    this._adns = adns;
}

TextDataNode.prototype = {
    _XHTMLNS: "http://www.w3.org/1999/xhtml",
    constructCell: function(cell, columnId, opened) {
        if (this.parent) {
            this._constructCell(cell, columnId, opened);
        } else {
            this._constructFragmentCell(cell, columnId, opened);
        }
    },
    _constructCell: function(cell, columnId, opened) {
        var div = document.createElementNS(this._XHTMLNS, "html:div");
        var className = "padded";
        
        if (columnId == "template") {
            div.appendChild(document.createTextNode(this._templateNode.sampleADN.nodeName));
        } else if (columnId == "field") {
            div.appendChild(document.createTextNode(
                (this._templateNode.fieldName) ? this._templateNode.fieldName : ""));
        } else if (columnId == "hits") {
            div.appendChild(document.createTextNode(this._templateNode.hits));
        } else if (columnId == "misses") {
            div.appendChild(document.createTextNode(this._templateNode.misses));
        } else {
            if (!opened) {
                var index = parseInt(columnId);
                var adn = this._adns[index];
                if (adn) {
                    var text = adn.text;
                    if (adn.nodeName == "@href" || adn.nodeName == "@src") {
                        var a = document.createElementNS(this._XHTMLNS, "html:a");
                        a.href = text;
                        a.target = "_blank";
                        a.appendChild(document.createTextNode(text));
                        div.appendChild(a);
                    } else {
                        div.appendChild(document.createTextNode(text.length > 100 ? (text.substr(0, 100) + "...") : text));
                    }
                    className += " limited-width";
                } else {
                    className += " missing";
                    div.appendChild(document.createTextNode("missing"));
                }
            }
        }
        div.className = className;
        cell.appendChild(div);
    },
    _constructFragmentCell: function(cell, columnId, opened) {
        if (columnId == "template") {
            var div = document.createElementNS(this._XHTMLNS, "html:div");
            div.className = "padded";
            
            div.appendChild(document.createTextNode("fragment"));
            cell.appendChild(div);
        }
    },
    get hasChildren() {
        return true;
    },
    get children() {
        if (!this._children) {
            this._expand();
        }
        return this._children;
    },
    _expand : function() {
        this._children = [];
        
        var bindingsArray = [];
        for (var j = 0; j < this._adns.length; j++) {
            if (this._adns[j]) {
                bindingsArray.push(this._templateNode.bindText(this._adns[j]));
            } else {
                bindingsArray.push(null);
            }
        }
        
        for (var i = 0; i < this._templateNode.children.length; i++) {
            var fragments = map(
                function(bindings) { 
                    return bindings ? (bindings[i] ? bindings[i].text : null) : null; 
                }, 
                bindingsArray
            );
            this._children.push(new TextFragmentDataNode(this._templateNode.children[i], fragments, this));
        }
    }
}

function TextFragmentDataNode(textTemplateNode, fragments, parentDataNode) {
    this.parent = parentDataNode;
    this._templateNode = textTemplateNode;
    this._fragments = fragments;
}

TextFragmentDataNode.prototype = {
    _XHTMLNS: "http://www.w3.org/1999/xhtml",
    constructCell: function(cell, columnId, opened) {
        var div = document.createElementNS(this._XHTMLNS, "html:div");
        var className = "padded";
        
        if (columnId == "template") {
            div.appendChild(document.createTextNode(this._templateNode.variable ? "variable" : "fixed"));
        } else if (columnId == "field") {
            div.appendChild(document.createTextNode(
                (this._templateNode.fieldName) ? this._templateNode.fieldName : ""));
        } else if (columnId == "hits") {
            div.appendChild(document.createTextNode(this._templateNode.hits));
        } else if (columnId == "misses") {
            div.appendChild(document.createTextNode(this._templateNode.misses));
        } else {
            var index = parseInt(columnId);
            var fragment = this._fragments[index];
            
            if (fragment != null) {
                div.appendChild(document.createTextNode(
                    fragment.length > 100 ? (fragment.substr(0, 100) + "...") : fragment));
                    
                if (this._templateNode.variable) {
                    className += " variable";
                } else {
                    className += " fixed";
                }
                
                className += " limited-width";
            } else {
                div.appendChild(document.createTextNode("missing"));
                className += " missing";
            }
        }
        div.className = className;
        
        cell.appendChild(div);
    },
    get hasChildren() {
        return false;
    },
    get children() {
        return null;
    }
}

