/*
 * Auto Scraper
 */

function AutoScraper() {
}

AutoScraper.prototype.scrape = function(
    browser,
    model,
    utilities,
    results,
    continuation
) {
    this.scrapeCount = 0;
    
    var scraper = this;
    var doc = browser.contentDocument;
    
    var namespace = doc.documentElement.namespaceURI;
    this.nsResolver = namespace ? function(prefix) {
        if (prefix == 'x') return namespace; else return null;
    } : null;
    
    var nodes = utilities.gatherElementsOnXPath(doc, doc, this.itemXpath, this.nsResolver);
    
    var uriBase = doc.location.href;
    (function() {
        var q = uriBase.indexOf("?");
        if (q > 0) {
            uriBase = uriBase.substr(0, q);
        }
        var a = uriBase.indexOf("&");
        if (a > 0) {
            uriBase = uriBase.substr(0, a);
        }
        var h = uriBase.lastIndexOf("#");
        if (h > 0) {
            uriBase = uriBase.substr(0, h);
        }
    })();
    
    var propertyBase = uriBase;
    (function() {
        if (propertyBase.substr(propertyBase.length - 1) != "/") {
            var slash = propertyBase.lastIndexOf("/");
            propertyBase = propertyBase.substr(0, slash + 1);
        }
    })();
    
    this.uriBase = uriBase;
    this.propertyBase = propertyBase;
    this.itemType = uriBase + "#Item";
    
    /*
     *  Set up results
     */
    results.dynamicQuery = DynamicQuery.createQuery(
        this.itemType, 
        "http://simile.mit.edu/2005/04/piggy-bank#original-order",
        true,
        nodes.length,
        0
    );
    results.augment = function(doc, items) { 
        scraper.augment(doc, items, utilities); 
    };
    
    /*
     *  Scrape continuation
     */
    var scrape = function(urls) {
        scraper.template.assignExtractableFields();
        
        utilities.processDocuments(browser, doc, urls, 
            function(d, cont, cancel) {
                scraper.scrapeAPage(d, model, utilities, results, cont, cancel);
            },
            function() { scraper.preparePage(continuation, doc, utilities); },
            function(e, url) {
                alert("Error extracting from " + url + "\n" + e);
            }
        );
    };
    
    /*
     *  Learn continuation
     */
    var learn = function(urls) {
        var learnURLs = [];
        for (var i = 2; i < urls.length; i += 3) {
            learnURLs.push(urls[i]);
        }
        
        scraper.template = Template.createTemplateFromItems(mapNew(Item, split(nodes, 1)));
        
        utilities.processDocuments(browser, null, learnURLs, 
            function(d, cont, cancel) {
                scraper.learnAPage(d, model, utilities, results, cont, cancel);
            },
            function() { scrape(urls) },
            function(e, url) {
                alert("Error learning from " + url + "\n" + e);
            }
        );
    }
    this.gatherPageURLs(doc, browser, utilities, learn);
}

AutoScraper.prototype.gatherPageURLs = function(doc, browser, utilities, cont) {
    var urls = [];
    var callCont = true;
    if (this.pagingData) {
        if (this.pagingData.heuristic == "LinkLabel") {
            var nodes = utilities.gatherElementsOnXPath(doc, doc, this.pagingData.xpath, this.nsResolver);
            for (var i = 0; i < nodes.length; i++) {
                var href = nodes[i].href;
                var n = NaN;
                var a = nodes[i].textContent.match(/\d+/);
                if (a instanceof Array) {
                    n = parseInt(a[0]);
                }
                
                if (href.length > 0 && !isNaN(n) && n > 1) {
                    var found = false;
                    for (var j = 0; j < urls.length; j++) {
                        if (urls[j] == href) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        urls.push(href);
                    }
                }
            }
        } else if (this.pagingData.heuristic == "URLParam") {
            var url = PiggyBankExplorePane._parseURL(doc.location.href);
            var pairs = [];
            for (var k = 0; k < url.pairs.length; k++) {
                var pair = url.pairs[k];
                if (pair.name != this.pagingData.name) {
                    pairs.push(pair.name + "=" + pair.value);
                }
            }
            
            pairs = pairs.join("&");
            
            var max = 0;
            var links = doc.links;
            for (var i = 0; i < links.length; i++) {
                var href = links[i].href;
                var x = href.indexOf(this.pagingData.name + "=");
                if (x > 0) {
                    x += this.pagingData.name.length + 1;
                    
                    var y = href.indexOf("&", x);
                    if (y < 0) {
                        y = href.length;
                    }
                    
                    var n = parseInt(href.substring(x, y));
                    if (!isNaN(n)) {
                        max = Math.max(max, n);
                    }
                }
            }
            
            var k = this.pagingData.startIndex + this.pagingData.diff;
            while (k <= max) {
                urls.push(url.base + "?" + pairs + "&" + this.pagingData.name + "=" + k);
                k += this.pagingData.diff;
            }
        } else if (this.pagingData.heuristic == "NextPage") {
            var scraper = this;
            var done = function() {
                if (cont) {
                    cont(urls);
                }
            }
            
            var nextPageURLs = scraper.findNextPageLinks(doc, 1);
            if (nextPageURLs.length == 0) {
                done();
            } else {
                var pageNumber = 2;
                var processor = function(doc, cont, cancel) {
                printDebug("next page " + doc.location.href);
                    urls.push(doc.location.href);
                    
                    var candidateURLs = scraper.findNextPageLinks(doc, pageNumber);
                    pageNumber++;
                    
                    cont(candidateURLs.length > 0 ? candidateURLs[0].url : null);
                };
                
                utilities.processDocuments2(
                    browser, 
                    nextPageURLs[0].url, 
                    processor,
                    done,
                    function(e, url) {
                        alert("Error learning from " + url + "\n" + e);
                    }
                );
            }
            
            callCont = false;
        }
    }
    
    if (callCont && (cont)) {
        cont(urls);
    }
    
    return urls;
}

AutoScraper.prototype.findDetectedPageLinks = function(doc, utilities) {
    if (this.pagingData.heuristic == "LinkLabel") {
        return utilities.gatherElementsOnXPath(doc, doc, this.pagingData.xpath, this.nsResolver);
    } else if (this.pagingData.heuristic == "URLParam") {
        var nodes = [];
        var links = doc.links;
        for (var i = 0; i < links.length; i++) {
            var node = links[i];
            var href = node.href;
            var x = href.indexOf(this.pagingData.name + "=");
            if (x > 0) {
                var a = node.textContent.match(/\d+/);
                if (a != null) {
                    nodes.push(links[i]);
                }
            }
        }
        return nodes;
    }
    return [];
}

AutoScraper.prototype.findNextPageLinks = function(doc, currentPageNumber) {
    var parseURL = function(url) {
        var segments = [];
        var i = 0;
        
        var indexOf = function(s, substring, start) {
            var x = s.indexOf(substring, start);
            return x < 0 ? s.length : x;
        }
        
        while (i < url.length) {
            var slash = indexOf(url, "/", i);
            var question = indexOf(url, "?", i);
            var ampersand = indexOf(url, "&", i);
            
            var j = Math.min(slash, Math.min(question, ampersand));
            var segment = url.substring(i, j);
            var a = segment.split("=");
            if (a != null && a.length > 1) {
                var n = parseInt(a[1]);
                if (isNaN(n)) {
                    segments.push({ name: a[0], value: 0 });
                } else {
                    segments.push({ name: a[0], value: n });
                }
            } else {
                segments.push({ name: segment, value: 0 }); 
            }
            i = j + 1;
        }
        return segments;
    };
    
    var compareURL = function(parsedURL1, parsedURL2) {
        var c = parsedURL1.length - parsedURL2.length;
        if (c != 0) {
            return c;
        }
        
        for (var i = 0; i < parsedURL1.length; i++) {
            var segment1 = parsedURL1[i];
            var segment2 = parsedURL2[i];
            c = segment1.name.length - segment2.name.length;
            if (c != 0) {
                return c;
            }
            
            c = segment1.name.localeCompare(segment2.name);
            if (c != 0) {
                return c;
            }
            
            c = segment1.value - segment2.value;
            if (c != 0) {
                return c;
            }
        }
        return 0;
    }
    
    var url = doc.location.href;
    var parsedURL = parseURL(url);
    var parsedURLs = [];
    
    var iterator = doc.evaluate(this.pagingData.xpath, doc, null, XPathResult.ANY_TYPE, null);
    var elmt = iterator.iterateNext();
    while (elmt) {
        var url2 = elmt.href;
        if (url2 != url) {
            var a = elmt.textContent.match(/\d+/);
            if (a == null || parseInt(a[0]) != currentPageNumber) {
                parsedURLs.push({ elmt: elmt, url: url2, parsedURL: parseURL(url2) });
            }
        }
        elmt = iterator.iterateNext();
    }
    
    parsedURLs.sort(function(a, b) {
        return compareURL(a.parsedURL, b.parsedURL);
    });
    
    var results = [];
    
    var i;
    for (i = 0; i < parsedURLs.length; i++) {
        if (compareURL(parsedURL, parsedURLs[i].parsedURL) < 0) {
            results.push(parsedURLs[i]);
            break;
        }
    }
    
    for (var j = i + 1; j < parsedURLs.length; j++) {
        if (compareURL(parsedURLs[i].parsedURL, parsedURLs[j].parsedURL) < 0) {
            break;
        }
        results.push(parsedURLs[j]);
    }
    
    return results;
}

AutoScraper.prototype.learnAPage = function(doc, model, utilities, results, cont, cancel) {
    if (this.canceled) {
        cancel();
        return;
    }
    
    try {
        var nodes = utilities.gatherElementsOnXPath(doc, doc, this.itemXpath, this.nsResolver);
        var items = mapNew(Item, split(nodes, 1));
        this.template.improveTemplateFromItems(items);
    } catch (e) {
        PB_Debug.onCaughtException(e);
    }
    cont();
}

AutoScraper.prototype._captureStyles = function(doc, elmt) {
    var style = doc.defaultView.getComputedStyle(elmt, "");
    elmt.style.border = style.border;
    elmt.style.margin = style.margin;
    elmt.style.padding = style.padding;
    elmt.style.background = style.background;
    elmt.style.color = style.color;
    elmt.style.font = style.font;
    elmt.style.fontFamily = style.fontFamily;
    elmt.style.fontSize = style.fontSize;
    elmt.style.fontStyle = style.fontStyle;
    elmt.style.fontVariant = style.fontVariant;
    elmt.style.fontWeight = style.fontWeight;
    elmt.style.textAlign = style.textAlign;
    elmt.style.textDecoration = style.textDecoration;
    elmt.style.textIndent = style.textIndent;
    elmt.style.textJustify = style.textJustify;
    elmt.style.textTransform = style.textTransform;
    elmt.style.verticalAlign = style.verticalAlign;
    
    for (var i = 0; i < elmt.childNodes.length; i++) {
        var node = elmt.childNodes[i];
        if (node.nodeType == 1) {
            this._captureStyles(doc, node);
        }
    }
}

AutoScraper.prototype.scrapeAPage = function(doc, model, utilities, results, cont, cancel) {
    if (this.canceled) {
        cancel();
        return;
    }
    
    var nodes = utilities.gatherElementsOnXPath(doc, doc, this.itemXpath, this.nsResolver);
    var items = mapNew(Item, split(nodes, 1));
    
    var propertyBase = this.propertyBase;
    
    for (var i = 0; i < items.length; i++) {
        var uri = this.uriBase + "#item-" + this.scrapeCount;
        
        var elementFixups = [];
        var textFixups = [];
        
        var extract = function(fieldName, fieldValue, domNode) {
            domNode = domNode.nodeType == 1 ? domNode : domNode.parentNode;
            elementFixups.push({ element : domNode, fieldName : fieldName, fieldValue : fieldValue });
        };
        
        var extractText = function(textFragments, fieldNames, fieldValues, domNode) {
            textFixups.push({ domNode : domNode, textFragments : textFragments, fieldNames : fieldNames, fieldValues : fieldValues });
        }
        
        var createSmartTag = function() {
            var elmt = doc.createElement("img");
            elmt.className = "piggyBank-smartTag";
            elmt.setAttribute("name", "piggyBank-smartTag");
            elmt.setAttribute("title", "Click to sort or refine by this field");
            elmt.setAttribute("onclick", "return AutoScraper.selectSmartTag(this, event);");
            elmt.setAttribute("onmouseover", "return AutoScraper.hoverSmartTag(this, event);");
            elmt.setAttribute("style", "display: none;");
            return elmt;
        }
        
        try {
            this.template.bind(items[i], extract, extractText);
            
            for (var j = 0; j < elementFixups.length; j++) {
                var fixup = elementFixups[j];
                var element = fixup.element;
                var propertyURI = propertyBase + fixup.fieldName;
                
                model.addStatement(uri, propertyURI, fixup.fieldValue, true);
                
                var propertyURIs = element.getAttribute("propertyURIs");
                if (!propertyURIs || propertyURIs.length == 0) {
                    propertyURIs = propertyURI;
                    
                    var tag = createSmartTag();
                    tag.setAttribute("propertyURIs", propertyURI);
                    
                    element.appendChild(tag);
                } else {
                    propertyURIs += "\n" + propertyURI;
                    element.lastChild.setAttribute("propertyURIs", propertyURIs);
                }
                element.setAttribute("propertyURIs", propertyURIs);
            }
            
            for (var j = 0; j < textFixups.length; j++) {
                var fixup = textFixups[j];
                var domNode = fixup.domNode;
                if (domNode.nodeType == 3) { // text
                    var text = "";
                    var parentNode = domNode.parentNode;
                    var nextSibling = domNode.nextSibling;
                    var insert = function(o) {
                        if (nextSibling) {
                            parentNode.insertBefore(o, nextSibling);
                        } else {
                            parentNode.appendChild(o);
                        }
                    }
                    
                    parentNode.removeChild(domNode);
                    
                    for (var x = 0; x < fixup.textFragments.length; x++) {
                        var fieldName = fixup.fieldNames[x];
                        var fieldText = fixup.textFragments[x];
                        var fieldValue = fixup.fieldValues[x] != null ? fixup.fieldValues[x] : fieldText;
                        
                        text += fieldText;
                        
                        if (fieldName && fieldName.length > 0) {
                            var propertyURI = propertyBase + fieldName;
                            model.addStatement(uri, propertyURI, fieldValue, true);
                            
                            var tag = createSmartTag();
                            tag.setAttribute("propertyURIs", propertyURI);
                            
                            insert(document.createTextNode(text));
                            insert(tag);
                            
                            text = "";
                        }
                    }
                    
                    if (text.length > 0) {
                        insert(document.createTextNode(text));
                    }
                } else {
                    var propertyURIs = [];
                    
                    for (var x = 0; x < fixup.textFragments.length; x++) {
                        var fieldName = fixup.fieldNames[x];
                        var fieldText = fixup.textFragments[x];
                        
                        if (fieldName && fieldName.length > 0) {
                            var propertyURI = propertyBase + fieldName;
                            model.addStatement(uri, propertyURI, fieldText, true);
                            
                            propertyURIs.push(propertyURI);
                        }
                    }
                    
                    if (propertyURIs.length > 0) {
                        propertyURIs = propertyURIs.join("\n");
                        
                        var tag = createSmartTag();
                        tag.setAttribute("propertyURIs", propertyURIs);
                        
                        domNode.appendChild(tag);
                    }
                }
            }
            
//            this._captureStyles(doc, nodes[i]);
            
            model.addStatement(uri, "http://www.w3.org/1999/02/22-rdf-syntax-ns#type", this.itemType, false);
            model.addStatement(uri, "http://simile.mit.edu/2005/04/piggy-bank#original-order", this.scrapeCount, true);
/*            
            var tagName = nodes[i].tagName;
            if (tagName == "TR") {
                model.addStatement(uri, "http://simile.mit.edu/2005/04/piggy-bank#innerHTML", 
                    "<table><tr>" + nodes[i].innerHTML + "</tr></table>", true);
            } else {
                model.addStatement(uri, "http://simile.mit.edu/2005/04/piggy-bank#innerHTML", nodes[i].innerHTML, true);
            }
*/            
            model.addStatement(uri, "http://simile.mit.edu/2005/04/piggy-bank#innerHTML", nodes[i].innerHTML, true);
        
            this.scrapeCount++;
        } catch (e) {
            PB_Debug.onCaughtException(e);
        }
    }
    
    cont();
}

AutoScraper.prototype.preparePage = function(cont, doc, utilities) {
    var style = doc.createElement("STYLE");
    style.innerHTML = 
        ".piggyBank-smartTag { " +
            "cursor: pointer; " +
            "border: none; " +
            "margin: 1px; " +
            "vertical-align: middle; " +
        "} ";
    doc.getElementsByTagName("HEAD").item(0).appendChild(style);
            
    var opacity = 0.3;
    var xpath = this.itemXpath;
    
    var nodes = utilities.gatherElementsOnXPath(doc, doc, xpath, this.nsResolver);
    while (nodes.length > 1) {
        xpath = xpath + "/..";
        
        var parentNodes = utilities.gatherElementsOnXPath(doc, doc, xpath, this.nsResolver);
        for (var i = 0; i < parentNodes.length; i++) {
            this._makeSiblingsTranslucent(parentNodes[i], nodes, opacity);
        }
        nodes = parentNodes;
    }
    
    this._makeAncestorSiblingsTranslucent(nodes[0], opacity);
    
    var script = doc.createElement("SCRIPT");
    script.type = "text/javascript";
    script.language = "JavaScript";
    script.src = "chrome://piggy-bank/content/scripts/auto-scraper-utilities.js";
    doc.getElementsByTagName("HEAD")[0].appendChild(script);
    
    cont();
}

AutoScraper.prototype._makeAncestorSiblingsTranslucent = function(node, opacity) {
    if (node.parentNode) {
        this._makeSiblingsTranslucent(node.parentNode, [ node ], opacity);
        this._makeAncestorSiblingsTranslucent(node.parentNode, opacity);
    }
}

AutoScraper.prototype._makeSiblingsTranslucent = function(parentNode, normalNodes, opacity) {
    var childNodes = parentNode.childNodes;
    for (var i = 0; i < childNodes.length; i++) {
        var childNode = childNodes[i];
        if (childNode.nodeType == 1) { // element
            var found = false;
            for (var j = 0; j < normalNodes.length; j++) {
                if (childNode == normalNodes[j]) {
                    found = true;
                    break;
                }
            }
            
            if (!found) {
                childNode.style.opacity = opacity;
            }
        }
    }
}

AutoScraper.prototype.augment = function(doc, items, utilities) {
    var augmentView = function(elmt, item) {
        if (elmt.tagName == "TD") {
            var row = elmt.parentNode;
            var index = elmt.cellIndex;
            var cell = row.insertCell(index);
            
            cell.className = elmt.className;
            cell.setAttribute("style", elmt.getAttribute("style"));
            cell.style.display = "table-cell";
            cell.innerHTML = item.innerHTML;
            
            row.removeChild(elmt);
        } else if (elmt.tagName == "TR") {
            var table = elmt.parentNode.parentNode;
            var index = elmt.rowIndex;
            var row = table.insertRow(index);
            
            row.className = elmt.className;
            row.setAttribute("style", elmt.getAttribute("style"));
            row.style.display = "table-row";
            row.innerHTML = item.innerHTML;
            
            elmt.parentNode.removeChild(elmt);
        } else {
            elmt.style.display = "block";
            elmt.innerHTML = item.innerHTML;
        }
    }
    
    var elmts = utilities.gatherElementsOnXPath(doc, doc, this.itemXpath, this.nsResolver);
    
    var i = 0;
    while (i < items.length) {
        augmentView(elmts[i], items[i]);
        i++;
    }
    while (i < elmts.length) {
        elmts[i].style.display = "none";
        i++;
    }
    
    var smartTags = doc.getElementsByName("piggyBank-smartTag");
    for (var i = 0; i < smartTags.length; i++) {
        var smartTag = smartTags[i];
        smartTag.setAttribute("src", "chrome://piggy-bank/skin/smart-tag.png");
        smartTag.style.display = "inline";
    }
    
    doc.defaultView.scrollTo(0,0);
}
