var PB_collector_count = 0;

function PB_Collector(tabBrowser, browser) {

    PB_Debug.trace("collector.js","> PB_Collector[" + PB_collector_count + "]");

    this._tabBrowser = tabBrowser;
    this._browser = browser;
    this._count = PB_collector_count++;

    PB_Debug.trace("collector.js","< PB_Collector");
};

PB_Collector.prototype.dispose = function() {
    PB_Debug.trace("collector.js","> PB_Collector.dispose[" + this._count +"]");

    this._tabBrowser = null;
    this._browser = null;
    PB_collector_count--;

    PB_Debug.trace("collector.js","< PB_Collector.dispose");
};

PB_Collector.prototype.loadDataForCurrentDocument = function() {
    PB_Debug.trace("collector.js","> PB_Collector.loadDataForCurrentDocument");

    var doc = this._browser.contentDocument;
    var url = doc.location.href;

    var sourceName = this._browser.getAttribute("piggyBank-sourceName");
    if (sourceName) {
        PB_Extension.getPiggyBankServer().setSource(sourceName, url);
    }

    if (url.indexOf("http://127.0.0.1") < 0) {
        var linkURLs = [ url ];
        var mimeTypes = [ doc.contentType ];
        var relations = [ "self" ];

        if (doc.getElementsByTagName) {
            try {
                var head = doc.getElementsByTagName("head")[0];
                if (head) { // sometimes documents have no 'head'
                    var links = head.getElementsByTagName("link");
                    for (var i = 0; i < links.length; i++) {
                        var link = links[i];

                        linkURLs.unshift(link.href);
                        mimeTypes.unshift(link.type);
                        relations.unshift(link.rel);
                    }
                }
            } catch (e) {
                PB_Debug.trace("collector.js","Error looking for <link/>: " + e);
            }
        }
        
        // NOTE: Firefox 1.1 and above consider "application/rdf+xml" their own 
        // territory and therefore do not trigger our own custom MIME type handler
        // but there is yet another problem, since RDF templates are used for
        // XUL-based applications, Firefox believes that an RDF/XML page is 
        // actually part of XUL, therefore the contentType of the document is
        // that of XUL. With this code, we check if the XUL is really it
        // or if it's RDF/XML and, in this case, we change the mimetype so
        // that the the profile gets loaded and the data coin shows up
        if (doc.contentType == "application/vnd.mozilla.xul+xml") {
            var root = doc.documentElement;
            if (root.localName == "RDF" && root.namespaceURI == "http://www.w3.org/1999/02/22-rdf-syntax-ns#") {
                mimeTypes = [ "application/rdf+xml" ];
            }
        }

        this._profile = PB_Extension.loadProfile(url, doc.title, linkURLs, mimeTypes, relations);
    } else {
        this._profile = null;
    }

    document.getElementById("piggyBank-datacoin").setAttribute("status","idle");

    this.updateWindow();
    
    PB_Debug.trace("collector.js","< PB_Collector.loadDataForCurrentDocument");
};

PB_Collector.prototype.loadDataFromURL = function(request) {

    var url = request.URI.spec;
    var mimeType = request.contentType;

    PB_Debug.trace("collector.js","> PB_Collector.loadDataFromURL(" + url + "," + mimeType + ")");

    var sourceName = this._browser.getAttribute("piggyBank-sourceName");
    if (sourceName) {
        PB_Extension.getPiggyBankServer().setSource(sourceName, url);
    }

    if (url.indexOf("http://127.0.0.1") < 0) {
        var linkURLs = [ url ];
        var mimeTypes = [ mimeType ];
        var relations = [ "self" ];

        this._profile = PB_Extension.loadProfile(url, url, linkURLs, mimeTypes, relations);
    } else {
        this._profile = null;
    }

    document.getElementById("piggyBank-datacoin").setAttribute("status","idle");

    this.updateWindow();

    PB_Debug.trace("collector.js","< PB_Collector.loadDataFromURL");
};

PB_Collector.prototype.hasData = function() {
    return (this._profile && this._profile.hasData())
};

PB_Collector.prototype.updateWindow = function(active) {
    PB_Debug.trace("collector.js","> PB_Collector.updateWindow");

    var didCollect = this.hasData();
    var datacoin = document.getElementById("piggyBank-datacoin");
    datacoin.collapsed = !didCollect;

	Pig.setStatus("status",PB_Extension.isRunning() ? "ready" : "disabled");
    PiggyBankExplorePane.onDataDetected(this._browser);

    PB_Debug.trace("collector.js","< PB_Collector.updateWindow");
};

PB_Collector.prototype.browseTidbits = function() {
    PB_Debug.trace("collector.js","> PB_Collector.browseTidbits");
    this.collectData(
        function(profileID, tabBrowser, browser, results) {
            PB_Extension.browseProfile(profileID, tabBrowser, browser);
        }
    );
    PB_Debug.trace("collector.js","< PB_Collector.browseTidbits");
};

PB_Collector.prototype.collectData = function(cont) {
    if (this._profile) {
        var profile = this._profile;
        var tabBrowser = this._tabBrowser;
        var browser = this._browser;
        var results = {};
        
        document.getElementById("piggyBank-datacoin").setAttribute("status","active");
        
        var scrapers = profile.getJavascriptScreenScrapers();
        if (scrapers.length > 0) {
            var piggyBank = PB_Extension.getPiggyBank();
            var scrapingUtilities = piggyBank.getScrapingUtilities();
            var model = scrapingUtilities.createWorkingModel();
            var debug = PB_Debug;
            
            var continuation = function(c, i) {
                if (i < scrapers.length) {
                    try {
                        var code = scrapers[i].getCacheCode();
                        if ("" == code) {
                            if (!window.confirm(
                                "Piggy Bank will need to retrieve code from\n" +
                                scrapers[i].getJavascriptURL() + "\n\nThis might take a bit of time.")) {
                                return false;
                            }
                            code = scrapers[i].getCode();
                        }

                        debug.trace("collector.js","> PB_Collector_executeJavascriptScraper");
                        PB_Collector_executeJavascriptScraper(
                            browser,
                            model,
                            scrapingUtilities,
                            results,
                            scrapers[i].getCode(),
                            function() { c(c, i + 1); }
                        );
                        debug.trace("collector.js","< PB_Collector_executeJavascriptScraper");

                    } catch (e) {
                        window.alert("Failed to extract data using screen scraper " + scrapers[i].getJavascriptURL() + "\nError: " + e);
                        model.dispose();
                    }
                } else {
                    profile.addScreenScrapedContent(model.detachRepository());
                    model.dispose();
                    
                    document.getElementById("piggyBank-datacoin").setAttribute("status","idle");
                    cont(profile.getID(), tabBrowser, browser, results);
                }
            }
            
            continuation(continuation, 0);
        } else {
            document.getElementById("piggyBank-datacoin").setAttribute("status","idle");
            cont(profile.getID(), tabBrowser, browser, results);
        }
        
        return true;
    } else {
        return false;
    }
}

PB_Collector.prototype.autoCollectData = function(scraper, cont) {
    if (this._profile) {
        var profile = this._profile;
        var tabBrowser = this._tabBrowser;
        var browser = this._browser;
        var results = {};
        
        document.getElementById("piggyBank-datacoin").setAttribute("status","active");
        
        var piggyBank = PB_Extension.getPiggyBank();
        var scrapingUtilities = piggyBank.getScrapingUtilities();
        var model = scrapingUtilities.createWorkingModel();
        var debug = PB_Debug;
        
        var continuation = function() {
            profile.addScreenScrapedContent(model.detachRepository());
            model.dispose();
            
            document.getElementById("piggyBank-datacoin").setAttribute("status","idle");
            cont(profile.getID(), tabBrowser, browser, results);
        };
        
        try {
            debug.trace("collector.js","> PB_Collector_autoCollectData");
            scraper.scrape(
                browser,
                model,
                new PB_ScrapingUtilities(),
                results,
                continuation
            );
            debug.trace("collector.js","< PB_Collector_autoCollectData");
        } catch (e) {
            window.alert("Failed to extract data using auto scraper\nError: " + e);
            model.dispose();
        }
        
        return true;
    } else {
        return false;
    }
}

function PB_Collector_executeJavascriptScraper(browser, model, scrapingUtilities, results, script, success) {

    var o = { wait: false };
    var doc = browser.contentDocument;
    var utilities = new PB_ScrapingUtilities();
    var wait = function () { o.wait = true; };
    var done = success;

    eval("" + script);

    if (!o.wait) {
        success();
    }
}
