var AddAccountDialog = {};

AddAccountDialog.startup = function() {
    var textbox = document.getElementById("bank-address-textbox");
    
    textbox.select();
    textbox.focus();
}

AddAccountDialog.shutdown = function() {
    if (AddAccountDialog._onStatus) {
        AddAccountDialog._onStatus.cancelled = true;
        AddAccountDialog._onStatus = null;
    }
    if (AddAccountDialog._onDone) {
        AddAccountDialog._onDone.cancelled = true;
        AddAccountDialog._onDone = null;
    }
}

AddAccountDialog.onOK = function() {
    try {
        AddAccountDialog._bankAddress = document.getElementById("bank-address-textbox").value;
        AddAccountDialog._accountID = document.getElementById("account-id-textbox").value;
        AddAccountDialog._password = document.getElementById("password-textbox").value;
        AddAccountDialog._emailAddress = document.getElementById("email-textbox").value;
        
        if (AddAccountDialog._bankAddress == "") {
            alert("You need to specify a bank address (e.g., http://localhost/bank).");
        } else if (AddAccountDialog._accountID == "") {
            alert("You need to specify an account ID (e.g., johnsmith).");
        } else if (AddAccountDialog._password == "") {
            alert("You need to enter a password.");
        } else {
            document.getElementById("ok-button").disabled = true;
            
            //AddAccountDialog._inquireService();
            AddAccountDialog._serviceUrl = AddAccountDialog._bankAddress;
            AddAccountDialog._acquireAccount();
        }
    } catch (e) {
        PB_Debug.onCaughtException();
    }
}

AddAccountDialog.onCancel = function() {
    setTimeout("close();", 100);
}

AddAccountDialog._inquireService = function() {
    document.getElementById("status-label").value = "Contacting bank...";
    
    AddAccountDialog._onStatus = function(status, statusText, xmlhttp) {
        if (!this.cancelled) {
            alert("Failed to inquire about semantic bank service from the address you gave.");
            AddAccountDialog._resetUI();
        }
    };
    AddAccountDialog._onDone = function(text, xmlhttp) {
        if (!this.cancelled) {
            try {
                AddAccountDialog._serviceUrl = xmlhttp.getResponseHeader("x-sembank-service");
                if ((AddAccountDialog._serviceUrl) && AddAccountDialog._serviceUrl != "") {
                    AddAccountDialog._acquireAccount();
                    return;
                }
            } catch (e) {
            }
            
            alert("There is no semantic bank service offered at the address you gave.");
            AddAccountDialog._resetUI();
        }       
    };
    
    PB_HTTPUtilities.doOptions(
        AddAccountDialog._bankAddress,
        "",
        null,
        AddAccountDialog._onDone);
}

AddAccountDialog._acquireAccount = function() {
    if (document.getElementById("already-registered-checkbox").checked) {
        AddAccountDialog._accountAcquired();
    } else {
        document.getElementById("status-label").value = "Acquiring account...";
        
        if (AddAccountDialog._serviceUrl.substr(AddAccountDialog._serviceUrl.length - 1) != "/") {
            AddAccountDialog._serviceUrl = AddAccountDialog._serviceUrl + "/";
        }
        
        AddAccountDialog._onStatus = function(status, statusText, xmlhttp) {
            if (!this.cancelled) {
                try {
                    if (status == 409) {
                        alert("Account ID already exists.");
                        AddAccountDialog._resetUI();
                    } else if (status == 201) {
                        AddAccountDialog._accountAcquired();
                    }
                } catch (e) {
                    PB_Debug.onCaughtException(e);
                }
            }
        };
        
        var xmlhttp = new XMLHttpRequest();
        
        xmlhttp.open('POST', AddAccountDialog._serviceUrl + AddAccountDialog._accountID + "?command=create", true);
        xmlhttp.overrideMimeType("text/xml");
        xmlhttp.setRequestHeader("x-sembank-password-hash", AddAccountDialog._hashPassword(AddAccountDialog._password));
        xmlhttp.setRequestHeader("x-sembank-email", AddAccountDialog._emailAddress);
        
        xmlhttp.onreadystatechange = function() {
            PB_HTTPUtilities_onReadyStateChange(
                xmlhttp, AddAccountDialog._onStatus, null);
        };
        xmlhttp.send("");
    }
}

AddAccountDialog._accountAcquired = function() {
    document.getElementById("status-label").value = "Registered account...";
    
    var localStore = 
        Components.classes["@mozilla.org/rdf/datasource;1?name=local-store"]
            .getService(Components.interfaces.nsIRDFDataSource);
    var rdfService = 
        Components.classes["@mozilla.org/rdf/rdf-service;1"]
            .getService(Components.interfaces.nsIRDFService);

    localStore.beginUpdateBatch();
    
    var accountResource = rdfService.GetAnonymousResource();
    //var accountResource = rdfService.GetResource(AddAccountDialog._serviceUrl);
    
    var typeProperty = rdfService.GetResource(RDF.type);
    var accountTypeResource = rdfService.GetResource(Simile.pb_Account);
    localStore.Assert(accountResource, typeProperty, accountTypeResource, true);
    
    var bankAddressProperty = rdfService.GetResource(Simile.pb_bankAddress);
    var bankAddressLiteral = rdfService.GetLiteral(AddAccountDialog._serviceUrl);
    localStore.Assert(accountResource, bankAddressProperty, bankAddressLiteral, true);
    
    var accountIDProperty = rdfService.GetResource(Simile.pb_accountID);
    var accountIDLiteral = rdfService.GetLiteral(AddAccountDialog._accountID);
    localStore.Assert(accountResource, accountIDProperty, accountIDLiteral, true);
    
    var passwordHash = AddAccountDialog._hashPassword(AddAccountDialog._password);
    var passwordHashProperty = rdfService.GetResource(Simile.pb_passwordHash);
    var passwordHashLiteral = rdfService.GetLiteral(passwordHash);
    localStore.Assert(accountResource, passwordHashProperty, passwordHashLiteral, true);
    
    PB_Extension.getPiggyBank().addAccount(
        AddAccountDialog._accountID, passwordHash, AddAccountDialog._serviceUrl);
    
    localStore.endUpdateBatch();
    
    setTimeout("close();", 100);
}

AddAccountDialog._resetUI = function() {
    document.getElementById("ok-button").disabled = false;
    document.getElementById("status-label").value = "";
}

AddAccountDialog._hashPassword = function(password) {
    return password;
}
