var BankAccountsDialog = {};

BankAccountsDialog.startup = function() {
    BankAccountsDialog._refreshList();
}

BankAccountsDialog.shutdown = function() {
}

BankAccountsDialog.add = function() {
    openDialog("chrome://piggy-bank/content/add-account-dialog.xul", "", "chrome,modal,width=300,height=280,centerscreen,dialog");
    BankAccountsDialog._refreshList();
}

BankAccountsDialog.remove = function() {
    var list = document.getElementById("existing-accounts");
    var item = list.selectedItem;
    if (!item) {
        return;
    }
        
    var accountURI = item.value;
    
    var localStore = 
        Components.classes["@mozilla.org/rdf/datasource;1?name=local-store"]
            .getService(Components.interfaces.nsIRDFDataSource);
    var rdfService = 
        Components.classes["@mozilla.org/rdf/rdf-service;1"]
            .getService(Components.interfaces.nsIRDFService);
            
    var typeProperty = rdfService.GetResource(RDF.type);
    var bankAddressProperty = rdfService.GetResource(Simile.pb_bankAddress);
    var accountIDProperty = rdfService.GetResource(Simile.pb_accountID);
    var passwordHashProperty = rdfService.GetResource(Simile.pb_passwordHash);
    
    var account = rdfService.GetResource(accountURI);
    var accountTypeResource = rdfService.GetResource(Simile.pb_Account);
    var bankAddress = localStore.GetTarget(account, bankAddressProperty, true).
        QueryInterface(Components.interfaces.nsIRDFLiteral);
    var accountID = localStore.GetTarget(account, accountIDProperty, true).
        QueryInterface(Components.interfaces.nsIRDFLiteral);
    var passwordHash = localStore.GetTarget(account, passwordHashProperty, true).
        QueryInterface(Components.interfaces.nsIRDFLiteral);
        
    if (confirm("Remove account " + accountID.Value + " at " + bankAddress.Value + " from your list (you can add it back later)?")) {
        localStore.beginUpdateBatch();
        
        localStore.Unassert(account, typeProperty, accountTypeResource);
        localStore.Unassert(account, bankAddressProperty, bankAddress);
        localStore.Unassert(account, accountIDProperty, accountID);
        localStore.Unassert(account, passwordHashProperty, passwordHash);
        
        PB_Extension.getPiggyBank().removeAccount(bankAddress.Value);
        
        localStore.endUpdateBatch();
        
        BankAccountsDialog._refreshList();
    }
}

BankAccountsDialog.changePassword = function() {
    openDialog("chrome://piggy-bank/content/change-password-dialog.xul", "", "chrome,modal,width=300,height=180,centerscreen,dialog");
}

BankAccountsDialog._refreshList = function() {
    var localStore = 
        Components.classes["@mozilla.org/rdf/datasource;1?name=local-store"]
            .getService(Components.interfaces.nsIRDFDataSource);
    var rdfService = 
        Components.classes["@mozilla.org/rdf/rdf-service;1"]
            .getService(Components.interfaces.nsIRDFService);
            
    var list = document.getElementById("existing-accounts");
    
    while (list.getRowCount() > 0) {
        list.removeItemAt(0);
    }

    var typeProperty = rdfService.GetResource(RDF.type);
    var accountTypeResource = rdfService.GetResource(Simile.pb_Account);
    var bankAddressProperty = rdfService.GetResource(Simile.pb_bankAddress);
    var accountIDProperty = rdfService.GetResource(Simile.pb_accountID);
    
    var accounts = localStore.GetSources(typeProperty, accountTypeResource, true);
    while (accounts.hasMoreElements()) {
        var account = accounts.getNext();
        var accountURI = account.Value;
        
        var bankAddress = localStore.GetTarget(account, bankAddressProperty, true).
            QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
        var accountID = localStore.GetTarget(account, accountIDProperty, true).
            QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
        
        var item = list.appendItem(
            bankAddress, 
            account.QueryInterface(Components.interfaces.nsIRDFResource).Value);
        
        var listcell;
        
        listcell = document.createElement("listcell");
        listcell.setAttribute("label", accountID);
        item.appendChild(listcell);
        
        listcell = document.createElement("listcell");
        listcell.setAttribute("label", bankAddress);
        item.appendChild(listcell);
    }
}
