var OptionsDialog = {};

OptionsDialog.startup = function() {
    PB_Debug.trace("options-dialog.js","> startup");

    // NOTE: we need to check if piggybank exist since we allow the options dialog
    // to be loaded even if piggybank failed to load!
    
    var pbURL = PB_Extension.getPiggyBankURL();
    if (pbURL) {
        var pburlbox = document.getElementById('personalPBURL');
        pburlbox.value = pbURL + "resources/content/panes/map/";
        pburlbox.select();
    }

    var port = PB_Extension.getPiggyBankPort();
    if (port) {
        var portbox = document.getElementById('port');
        portbox.value = port;
    }

    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    if (prefs.getPrefType(PB_Preferences.googleKey) == prefs.PREF_STRING) {
        var googleKey = prefs.getCharPref(PB_Preferences.googleKey);
        PB_Debug.trace("options-dialog.js","google key found: " + googleKey);
        document.getElementById('googlekey').value = googleKey;
    }
    if (prefs.getPrefType(PB_Preferences.debug_classes_longwell) == prefs.PREF_STRING) {
        var longwellClasses = prefs.getCharPref(PB_Preferences.debug_classes_longwell);
        PB_Debug.trace("options-dialog.js","longwell classes found: " + longwellClasses);
        document.getElementById('longwellClasses').value = longwellClasses;
    }
    if (prefs.getPrefType(PB_Preferences.debug_classes_piggybank) == prefs.PREF_STRING) {
        var piggybankClasses = prefs.getCharPref(PB_Preferences.debug_classes_piggybank);
        PB_Debug.trace("options-dialog.js","piggybank classes found: " + piggybankClasses);
        document.getElementById('piggybankClasses').value = piggybankClasses;
    }
    
    if (prefs.getPrefType(PB_Preferences.debug) == prefs.PREF_BOOL) {
        var debug = prefs.getBoolPref(PB_Preferences.debug);
        PB_Debug.trace("options-dialog.js","debug found: " + debug);
        document.getElementById("debug").checked = debug;
        OptionsDialog.debugDisabled(!debug);
    }
    if (prefs.getPrefType(PB_Preferences.debug_redirectJSConsole) == prefs.PREF_BOOL) {
        var redirectJSConsole = prefs.getBoolPref(PB_Preferences.debug_redirectJSConsole);
        PB_Debug.trace("options-dialog.js","redirect JS console found: " + redirectJSConsole);
        document.getElementById("redirectJSConsole").checked = redirectJSConsole;
    }
    if (prefs.getPrefType(PB_Preferences.debug_tracing_javascript) == prefs.PREF_BOOL) {
        var javascriptTracing = prefs.getBoolPref(PB_Preferences.debug_tracing_javascript);
        PB_Debug.trace("options-dialog.js","javascript tracing found: " + javascriptTracing);
        document.getElementById("javascriptTracing").checked = javascriptTracing;
    }
    if (prefs.getPrefType(PB_Preferences.debug_tracing_classloader) == prefs.PREF_BOOL) {
        var classloaderTracing = prefs.getBoolPref(PB_Preferences.debug_tracing_classloader);
        PB_Debug.trace("options-dialog.js","classloader tracing found: " + classloaderTracing);
        document.getElementById("classloaderTracing").checked = classloaderTracing;
    }
    if (prefs.getPrefType(PB_Preferences.debug_tracing_java) == prefs.PREF_BOOL) {
        var javaTracing = prefs.getBoolPref(PB_Preferences.debug_tracing_java);
        PB_Debug.trace("options-dialog.js","java tracing found: " + javaTracing);
        document.getElementById("javaTracing").checked = javaTracing;
    }

    PB_Debug.trace("options-dialog.js","< startup");
};

OptionsDialog.shutdownAccept = function() {
    PB_Debug.trace("options-dialog.js","> shutdownAccept");
    
    var log = Components.classes["@simile.mit.edu/logger;1"].getService(Components.interfaces.nsILogger);
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    // parameters that DON'T need restarting ---------------------------------------
    
    // get debug status
    var debug = document.getElementById('debug').checked;
    PB_Debug.trace("options-dialog.js","debug: " + debug);
    prefs.setBoolPref(PB_Preferences.debug, debug);    
    log.enable(debug);
    
    // Make sure that the pig toolbar icon (if present!) is in debug mode
    Pig.setStatus("debug",debug);

    // get redirect jsconsole status
    var redirectJSConsole = document.getElementById('redirectJSConsole').checked;
    PB_Debug.trace("options-dialog.js","redirect JS console: " + redirectJSConsole);
    prefs.setBoolPref(PB_Preferences.debug_redirectJSConsole, redirectJSConsole);    
    log.redirectJSConsole(redirectJSConsole && debug);

    // get javascript tracing status
    var javascriptTracing = document.getElementById('javascriptTracing').checked;
    PB_Debug.trace("options-dialog.js","javascript tracing: " + javascriptTracing);
    prefs.setBoolPref(PB_Preferences.debug_tracing_javascript, javascriptTracing);    
    log.enableTracing(javascriptTracing && debug);

    // get classloading tracing status
    var classloaderTracing = document.getElementById('classloaderTracing').checked;
    PB_Debug.trace("options-dialog.js","classloader tracing: " + classloaderTracing);
    prefs.setBoolPref(PB_Preferences.debug_tracing_classloader, classloaderTracing);
    PB_Extension.setClassloaderTracing(classloaderTracing && debug);

    // get java tracing status
    var javaTracing = document.getElementById('javaTracing').checked;
    PB_Debug.trace("options-dialog.js","java tracing: " + javaTracing);
    prefs.setBoolPref(PB_Preferences.debug_tracing_java, javaTracing);    
    PB_Extension.setJavaTracing(javaTracing && debug);
    
    // get google key
    var key = document.getElementById('googlekey').value;
    PB_Debug.trace("options-dialog.js","google key: " + key);
    prefs.setCharPref(PB_Preferences.googleKey, key);
    PB_Extension.setGoogleKey(key);

    // parameters that need restarting --------------------------------------------

    var portvalue = document.getElementById('port').value;
    if (portvalue) {
        var port = OptionsDialog.testPort(portvalue);
        if (port < 0) {
            return false;
        } else {
            PB_Debug.trace("options-dialog.js","port: " + port);
            prefs.setIntPref(PB_Preferences.port, port);
        }
    }

    var longwellClasses = document.getElementById('longwellClasses').value
    PB_Debug.trace("options-dialog.js","longwell classes: " + longwellClasses);
    prefs.setCharPref(PB_Preferences.debug_classes_longwell, longwellClasses);
    
    var piggybankClasses = document.getElementById('piggybankClasses').value
    PB_Debug.trace("options-dialog.js","piggybank classes: " + piggybankClasses);
    prefs.setCharPref(PB_Preferences.debug_classes_piggybank, piggybankClasses);
    
    PB_Debug.trace("options-dialog.js","< shutdownAccept");
    
    return true;
};

OptionsDialog.debugDisabled = function(disabled) {
    document.getElementById("redirectJSConsole").disabled = disabled;
    document.getElementById("javascriptTracing").disabled = disabled;
    document.getElementById("classloaderTracing").disabled = disabled;
    document.getElementById("javaTracing").disabled = disabled;
}

OptionsDialog.testPort = function(port) {
    try {
        var intport = parseInt(port);
        document.getElementById("portError").style.display = 'none'; 
        return intport;
    } catch (e) {
        document.getElementById("portError").style.display = 'inline'; 
        return -1;
    }
};

OptionsDialog.link = function(anchor) {
    window.open(anchor.getAttribute("link"));
};
