/*
 * Dynamic Query
 */

var DynamicQuery = new Object();

DynamicQuery.createQuery = function(itemTypeURI, sortPropertyURI, sortAscending, itemsPerPage, pageIndex) {
    return {
        itemTypeURI:        itemTypeURI,
        sortPropertyURI:    sortPropertyURI,
        sortAscending:      sortAscending,
        itemsPerPage:       itemsPerPage,
        pageIndex:          pageIndex,
        facetInfos:         []
    };
}

DynamicQuery.duplicateQuery = function(query) {
    return {
        itemTypeURI:        query.itemTypeURI,
        sortPropertyURI:    query.sortPropertyURI,
        sortAscending:      query.sortAscending,
        itemsPerPage:       query.itemsPerPage,
        pageIndex:          query.pageIndex,
        facetInfos:         [].concat(query.facetInfos)
    };
};

DynamicQuery.writeXML = function(query, easyRead) {
    var s = "<query" +
        " itemTypeURI=\"" + query.itemTypeURI + "\"" +
        " sortPropertyURI=\"" + query.sortPropertyURI + "\"" +
        " sortAscending=\"" + query.sortAscending + "\"" +
        " itemsPerPage=\"" + query.itemsPerPage + "\"" +
        " pageIndex=\"" + query.pageIndex + "\"" +
        ">";
    if (easyRead) {
        s += "\n";
    }
        
    for (var i = 0; i < query.facetInfos.length; i++) {
        s += DynamicQuery._writeFacetInfoXML(query.facetInfos[i], easyRead, " ");
    }
        
    s += "</query>";
    return s;
}

DynamicQuery._writeFacetInfoXML = function(facetInfo, easyRead, indent) {
    var s = "<facetInfo" +
        " propertyURI=\"" + facetInfo.propertyURI + "\"" +
        " facetType=\"" + (facetInfo.facetType == "range" ? 0 : 1) + "\"" +
        " valueClass=\"" + facetInfo.valueClass + "\"" +
        " isTarget=\"" + ((facetInfo.isTarget) ? "true" : "false") + "\"" +
        ">";
    if (easyRead) {
        s = indent + s + "\n";
    }
    
    for (var i = 0; i < facetInfo.facetValues.length; i++) {
        s += DynamicQuery._writeFacetValueXML(facetInfo.facetValues[i], easyRead, indent + " ");
    }
    
    if (easyRead) {    
        s += indent + "</facetInfo>\n";
    } else {    
        s += "</facetInfo>";
    }
    return s;
}

DynamicQuery._writeFacetValueXML = function(facetValue, easyRead, indent) {
    var s = "<facetValue" +
        " value=\"" + DynamicQuery._encodeValue(facetValue.value) + "\"" +
        " value2=\"" + DynamicQuery._encodeValue(facetValue.value2) + "\"" +
        " selected=\"" + ((facetValue.selected) ? "true" : "false") + "\"" +
        " expandable=\"" + ((facetValue.expandable) ? "true" : "false") + "\"" +
        " isTarget=\"" + ((facetValue.isTarget) ? "true" : "false") + "\"" +
        " isOpen=\"" + ((facetValue.isOpen) ? "true" : "false") + "\"" +
        " status=\"" + facetValue.status + "\"" +
        ">";
    if (easyRead) {
        s = indent + s + "\n";
    }
    
    if (facetValue.facetValues) {
        for (var i = 0; i < facetValue.facetValues.length; i++) {
            s += DynamicQuery._writeFacetValueXML(facetValue.facetValues[i], easyRead, indent + " ");
        }
    }

    if (easyRead) {    
        s += indent + "</facetValue>\n";
    } else {    
        s += "</facetValue>";
    }
    return s;
}

DynamicQuery._encodeValue = function(v) {
    return v == null ? "null" : v.replace(/\&/g, "&#38;").replace(/\"/g, "&#34;");
}