function FacetTreeItem(facetValue, level) {
    this.level = level;
    this.isContainer = facetValue.expandable;
    this.facetValue = facetValue;
}

function FacetTreeView(facetInfo, id) {
    var rows = [];
    
    var f = function(facetValue, level, recurse) {
        var item = new FacetTreeItem(facetValue, level);
        rows.push(item);
        
        if (facetValue.isOpen && "facetValues" in facetValue) {
            item.childRows = []
            for (var i = 0; i < facetValue.facetValues.length; i++) {
                item.childRows.push(recurse(facetValue.facetValues[i], level + 1, recurse));
            }
        }
        return item;
    }
    
    this.childRows = [];
    for (var i = 0; i < facetInfo.facetValues.length; i++) {
        this.childRows.push(f(facetInfo.facetValues[i], 0, f));
    }
    
    this._id = id;
    this._facetInfo = facetInfo;
    
    this._rows = rows;
    this._treeBox = null;
}

FacetTreeView.prototype = {
    setTree : function(treeBox) {
        this._treeBox = treeBox;
    },
    
    isContainer : function(row) {
        return this._rows[row].isContainer;
    },
    isContainerOpen : function(row) {
        return this._rows[row].facetValue.isOpen;
    },
    isContainerEmpty : function(row) {
        return false;
    },
    get rowCount() {
        return this._rows.length;
    },
    getCellText : function(row, col) {
        var id = this._getColumnId(col);
        var facetValue = this._rows[row].facetValue;
        
        if (id == "label")
            return facetValue.label;
        else if (id == "count")
            return facetValue.count;
        else
            return "";
    },
    setCellText : function(row, col, text) {
    },
    getParentIndex : function(row) {
        var level = this.getLevel(row);
        for (var r = row - 1; r >= 0; r--) {
            if (this.getLevel(r) < level)
                return r;
        }
        return -1;
    },
    getLevel : function(row) {
        return this._rows[row].level;
    },
    hasNextSibling : function(row, after) {
        var level = this.getLevel(row);
        for (var r = after + 1; r < this._rows.length; r++) {
            var level2 = this.getLevel(r);
            if (level2 == level)
                return true;
            else if (level2 < level)
                break;
        }
        return false;
    },
    
    
    toggleOpenState : function(row) {
        if (this.isContainerEmpty(row))
            return;
            
        this.updating = true;
        
        var item = this._rows[row];
        if (item.facetValue.isOpen) {
            UserStudy.log("user\tcollapse-facet-value\t" + this._facetInfo.propertyURI);
            
            var level = this.getLevel(row);
            
            var selected = false;
            var selection = this._treeBox.view.selection;
            for (var r = row + 1; r < this._rows.length; r++) {
                if (this.getLevel(r) > level) {
                    if (selection.isSelected(r)) {
                        selected = true;
                        break;
                    }
                } else {
                    break;
                }
            }
            
            if (!selected) {
                // closing item, deleting descendents
                var deleteCount = 0;
                for (var r = row + 1; r < this._rows.length; r++) {
                    if (this.getLevel(r) > level) {
                        deleteCount++;
                        this._rows[r].facetValue.isOpen = false;
                    } else {
                        break;
                    }
                }
                
                if (deleteCount > 0) {
                    this._rows.splice(row + 1, deleteCount);
                    this._treeBox.rowCountChanged(row + 1, -deleteCount);
                }
                
                item.facetValue.isOpen = false;
                this._treeBox.invalidateRow(row);
            }
        } else {
            UserStudy.log("user\texpand-facet-value\t" + this._facetInfo.propertyURI);
            
            var newRows = this._rows[row].childRows;
            if (!newRows) {
                this._rows[row].childRows = [];
                
                this._facetInfo.isTarget = true;
                item.facetValue.isTarget = true;
                
                item.facetValue.isOpen = true;
                this._treeBox.invalidateRow(row);
                
                PiggyBankExplorePane.expandTargetedFacetValues(this._id);
            } else {
                this._rows = this._rows.slice(0, row + 1).concat(newRows).concat(this._rows.slice(row + 1));
                this._treeBox.rowCountChanged(row + 1, newRows.length);
                
                var selection = this._treeBox.view.selection;
                for (var x = 0; x < newRows.length; x++) {
                    var facetValue = newRows[x].facetValue;
                    var actualRow = row + 1 + x;
                    if (facetValue.selected && !selection.isSelected(actualRow)) {
                        selection.toggleSelect(actualRow);
                    }
                }
                
                item.facetValue.isOpen = true;
                this._treeBox.invalidateRow(row);
            }
        }
        this.updating = false;
    },
    
    
    isEditable :          function(row, col) { return false; },
    isSeparator :         function(row) { return false; },
    isSorted :            function(row) { return false; },
    getImageSrc :         function(row, col) {},
    getProgressMode :     function(row, col) {},
    getCellValue :        function(row, col) {},
    cycleHeader :         function(colID, elmt) {},
    cycleCell :           function(row, col) {},
    getRowProperties :    function(row, props) {},
    getCellProperties :   function(row, col, props) {},
    getColumnProperties : function(colid, col, props) {},
    selectionChanged :    function() {},
    performAction :       function(action) {},
    performActionOnCell : function(action, row, col) {},
    
    _getColumnId : function(col) {
        return ((typeof col) == "string") ? col : col.id;
    }
}

FacetTreeView.prototype.update = function(facetInfo) {
    this.updating = true;
    
    try {
        this._merge(this, facetInfo.facetValues, 0, 0, true);
    } catch (e) {
        printDebug(e);
    }
    this._facetInfo = facetInfo;
    
    this.updating = false;
}

FacetTreeView.prototype._merge = function(o, newFacetValues, level, visibleRowIndex, visible) {
    var selection = this._treeBox.view.selection;
    var childRows = o.childRows;
    var selectedAsParent = childRows.length == 0;
    
    var oldI = 0;
    var newI = 0;
    
    while (newI < newFacetValues.length) {
        var newFacetValue = newFacetValues[newI];
        var item = null;
        
        var status = newFacetValue.status;
        if (status == "added") {
            if (selectedAsParent && (o.selected)) {
                newFacetValue.selected = true;
            }
            
            item = new FacetTreeItem(newFacetValue, level);
            
            childRows.splice(oldI, 0, item);
            oldI++;
            newI++;
            
            if (visible) {
                this._rows.splice(visibleRowIndex, 0, item);
                this._treeBox.rowCountChanged(visibleRowIndex, 1);
                
                visibleRowIndex++;
            }
        } else if (status == "removed") {
            childRows.splice(oldI, 1);
            newFacetValues.splice(newI, 1);
            
            if (visible) {
                var deleteCount = 0;
                for (var r = visibleRowIndex + 1; r < this._rows.length; r++) {
                    if (this.getLevel(r) > level) {
                        deleteCount++;
                    } else {
                        break;
                    }
                }
                
                deleteCount++; // include the current item, too
            
                this._rows.splice(visibleRowIndex, deleteCount);
                this._treeBox.rowCountChanged(visibleRowIndex, -deleteCount);
            }
        } else {
            item = childRows[oldI];
            
            if (newFacetValue.value != item.facetValue.value) {
                printDebug("-----error: update mismatch----- " + newI + " " + oldI);
                inspect(childRows);
                inspect(newFacetValues);
                return;
            }
            
            newFacetValue.label = item.facetValue.label;
            item.facetValue = newFacetValue;
            oldI++;
            newI++;
            
            if (visible) {
                this._treeBox.invalidateRow(visibleRowIndex);
                
                visibleRowIndex++;
            }
        }
        
        if ((item != null) && (newFacetValue.facetValues)) {
            visibleRowIndex = this._merge(
                item, 
                newFacetValue.facetValues, 
                level + 1, 
                visibleRowIndex, 
                visible && item.facetValue.isOpen
            );
        }
    }
    
    return visibleRowIndex;
}

FacetTreeView.prototype.reselect = function() {
    this.updating = true;
    
    var selection = this._treeBox.view.selection;
    for (var i = 0; i < this._rows.length; i++) {
        var item = this._rows[i];
        if (selection.isSelected(i) != item.facetValue.selected) {
            selection.toggleSelect(i);
        }
    }
    
    this.updating = false;
}