PiggyBankExplorePane._parseURL = function(url) {
    var question = url.indexOf("?");
    if (question < 0) {
        return { base: url, pairs: [] };
    } else {
        var base = url.substr(0, question);
        var pairs = [];
        
        var a = url.substr(question+1).split("&");
        for (var i = 0; i < a.length; i++) {
            var pair = a[i].split("=");
            var name = pair[0];
            var value = pair.length > 1 ? pair[1] : "";
            pairs.push({ name: name, value: value });
        }
        
        return { base: base, pairs: pairs };
    }
}

PiggyBankExplorePane._detectPagesURLParamHeuristic = function(entry, wizard, doc) {
    var paramNameToValues = [];
    var links = doc.links;
    for (var i = 0; i < links.length; i++) {
        var href = links[i].href;
        var url = PiggyBankExplorePane._parseURL(href);
        for (var k = 0; k < url.pairs.length; k++) {
            var pair = url.pairs[k];
            var n = parseInt(pair.value);
            if (!isNaN(n)) {
                var label = NaN;
                var a = links[i].innerHTML.match(/\d+/);
                if (a instanceof Array) {
                    label = parseInt(a[0]);
                }
                
                var values = paramNameToValues[pair.name];
                if (!(values instanceof Array)) {
                    values = [];
                    paramNameToValues[pair.name] = values;
                }
                values.push({ value: n, label: label });
            }
        }
    }
    
    var candidates = [];
    for (name in paramNameToValues) {
        /*
         *  Sort and condense
         */
        var values = paramNameToValues[name].sort(function(a, b) { return a.value - b.value; });
        
        var values2 = [];
        var n = values[0];
        for (var x = 1; x < values.length; x++) {
            if (n.value != values[x].value) {
                values2.push(n);
                n = values[x];
            }
        }
        values2.push(n);
        paramNameToValues[name] = values2;
            
        if (values2.length > 2) {
            /*
             *  See if there is a consistent difference between consecutive numbers
             */
            var diff = values2[1].value - values2[0].value;
            var consistent = true;
            for (var x = 2; x < values2.length && x < 3; x++) {
                if (values2[x].value - values2[x-1].value != diff) {
                    consistent = false;
                    break;
                }
            }
            
            if (consistent && diff >= 1) {
                /*
                 *  Try to determine whether index starts at 0 or 1. This will
                 *  help us eliminate the duplicated first page.
                 */
                var startIndex = 0;
                for (var i = 0; i < values.length; i++) {
                    var value = values[i].value;
                    var label = values[i].label;
                    if (!isNaN(values[i].label)) {
                        if (diff == 1) { // parameter encodes page index
                            if (label == value) {
                                startIndex = 1;
                                break;
                            } else if (label == value + 1) {
                                startIndex = 0;
                                break;
                            }
                        } else { // parameter encodes item index
                            if ((label - 1) * diff == value) {
                                startIndex = 0;
                                break;
                            } else if ((label - 1) * diff == value - 1) {
                                startIndex = 1;
                                break;
                            } else if (label == value) {
                                startIndex = 1;
                                break;
                            } else if (label == value + 1) {
                                startIndex = 0;
                                break;
                            }
                        }
                    }
                }
                
                candidates.push({ 
                    name:       name, 
                    diff:       diff, 
                    startIndex: startIndex,
                    count:      1 + (values2[values2.length - 1].value - startIndex) / diff 
                });
            }
        }
    }
    
    if (candidates.length > 0) {
        candidates.sort(function(a, b){ 
            var countDiff = b.count - a.count;
            return countDiff != 0 ? countDiff : b.diff - a.diff; 
        });
        
        var candidate = candidates[0];
        
        entry.scraper.pagingData = { 
            name:       candidate.name,
            diff:       candidate.diff,
            startIndex: candidate.startIndex,
            heuristic:  "URLParam"
        };
        
        entry.pageCount = candidate.count;
        
        wizard.setPageCount(candidate.count);
        wizard.setTotalItemCountText(
            (entry.itemCount * (candidate.count - 1)) + " ~ " + (entry.itemCount * candidate.count)
        );
        wizard.showSummaryPage(0);
        
        UserStudy.log("system\tdetected-pages\tURLParam\t" + 
            candidate.name + "\t" + 
            candidate.diff + "\t" + 
            candidate.startIndex
        );
        
        return true;
    } else {
        return false;
    }
}

PiggyBankExplorePane._detectPagesLinkLabelHeuristic = function(entry, wizard, doc) {
    var xpathToValues = [];
    var links = doc.links;
    for (var i = 0; i < links.length; i++) {
        var link = links[i];
        var n = NaN;
        
        var a = link.textContent.match(/\d+/);
        if (a instanceof Array) {
            n = parseInt(a[0]);
        }
        if (!isNaN(n)) {
            var xpath = "";
            var node = link;
            while (node != null && node != doc) {
                var options = "";
                if (node.className && node.className.length > 0) {
                    options = "[@class='" + node.className + "']";
                }
                
                xpath = "/" + node.tagName + options + xpath;
                node = node.parentNode;
            }
            
            var values = xpathToValues[xpath];
            if (!(values instanceof Array)) {
                values = [];
                xpathToValues[xpath] = values;
            }
            values.push(n);
        }
    }
    
    var candidates = [];
    for (xpath in xpathToValues) {
        /*
         *  Sort and condense
         */
        var values = xpathToValues[xpath].sort(function(a, b) { return a - b; });
        
        var values2 = [];
        var n = values[0];
        for (var x = 1; x < values.length; x++) {
            if (n != values[x]) {
                values2.push(n);
                n = values[x];
            }
        }
        values2.push(n);
        xpathToValues[name] = values2;
            
        if (values2.length > 2) {
            /*
             *  See if there is a consistent difference between consecutive numbers
             */
            var diff = values2[1] - values2[0];
            var consistent = true;
            for (var x = 2; x < values2.length; x++) {
                if (values2[x] - values2[x-1] != diff) {
                    consistent = false;
                    break;
                }
            }
            
            if (consistent) {
                candidates.push({ 
                    xpath:  xpath, 
                    diff:   diff, 
                    values: values2,
                    count:  values2.length 
                });
            }
        }
    }
    
    if (candidates.length > 0) {
        candidates.sort(function(a, b){ 
            var countDiff = b.count - a.count;
            return countDiff != 0 ? countDiff : b.diff - a.diff; 
        });
        
        var candidate = candidates[0];
        
        entry.scraper.pagingData = {
            xpath:      candidate.xpath,
            heuristic:  "LinkLabel"
        };
        
        var pageCount = candidate.diff > 1 ?
            ((candidate.values[candidate.values.length - 1]) - 1) / candidate.diff + 1 :
            (candidate.values[candidate.values.length - 1]);
        
        entry.pageCount = pageCount;
        
        wizard.setPageCount(pageCount);
        wizard.setTotalItemCountText(
            (entry.itemCount * (pageCount - 1)) + " ~ " + (entry.itemCount * pageCount)
        );
        wizard.showSummaryPage(0);
        
        UserStudy.log("system\tdetected-pages\tLinkLabel\t" + 
            candidate.xpath + "\t" + 
            candidate.diff
        );
        
        return true;
    } else {
        return false;
    }
}

