/*
 * The Piggy Bank Explore Pane, scraping aspect
 */
function printDebug(msg) {
    Components.classes["@mozilla.org/consoleservice;1"]
        .getService(Components.interfaces.nsIConsoleService)
            .logStringMessage(msg);
}

/*----------------------------------------------------------------------
 *  Item
 *----------------------------------------------------------------------
 */
 
function Item(domNodes) {
    this.children = map(
        function(domNode) {
            return new AbstractDomNode(domNode, true);
        },
        domNodes
    );
}

Item.prototype.createTemplate = function() {
    return new Template(
        map(
            function(adnNode) {
                return TemplateNode.createTemplateNode(adnNode);
            },
            this.children
        )
    );
}

/*----------------------------------------------------------------------
 *  PiggyBankExplorePane
 *----------------------------------------------------------------------
 */

PiggyBankExplorePane._scrapeResetUI = function(pane, entry) {
    var wizard = pane.childNodes[0];
    wizard.reset();
    
    wizard.setBrowseSortHandler(PiggyBankExplorePane.browseSort);
    wizard.setLocateItemsHandler(PiggyBankExplorePane.locateItems);
    wizard.setLocatePagesHandler(PiggyBankExplorePane.locatePages);
    wizard.setDetectPagesHandler(PiggyBankExplorePane.detectPages);
    wizard.setItemCountLinkHandler(PiggyBankExplorePane.showItems);
    wizard.setNextPageLinkHandler(PiggyBankExplorePane.showNextPageLink);
    wizard.setDetectedPageLinksHandler(PiggyBankExplorePane.showDetectedPageLinks);

    wizard.setStartExploringHandler(PiggyBankExplorePane.startExploring);
    wizard.setStopExploringHandler(PiggyBankExplorePane.stopExploring);
}

PiggyBankExplorePane._reapplyScraperSettings = function(pane, entry) {
    var wizard = pane.childNodes[0];
    var doc = entry.browser.contentDocument;
    if (entry.scraper.itemXpath) {
        entry.itemCount = PiggyBankExplorePane._countXpath(doc, entry.scraper.itemXpath);
        wizard.setItemCount(entry.itemCount);
        
        if (entry.scraper.pagingData) {
            if (entry.scraper.pagingData.heuristic == "NextPage") {
                wizard.showSummaryPage(2);
            } else if (entry.scraper.pagingData.heuristic == "NoAdditionalPages") {
                wizard.showSummaryPage(3);
            } else {
                var pageCount = 1 + entry.scraper.gatherPageURLs(
                    doc, entry.browser, new PB_ScrapingUtilities(), function(){}).length;
                
                wizard.setPageCount(pageCount);
                wizard.setTotalItemCountText(
                    (entry.itemCount * (pageCount - 1)) + " ~ " + (entry.itemCount * pageCount)
                );
                wizard.showSummaryPage(0);
            }
        } else {
            PiggyBankExplorePane.detectPages(wizard);
        }
    }
}

PiggyBankExplorePane.detectPages = function(wizard) {
    UserStudy.log("user\tdetect-pages\t" + wizard.getSummaryPage());
    PiggyBankExplorePane._detectPages(PiggyBankExplorePane._registry[wizard.parentNode.id]);
}

PiggyBankExplorePane._detectPages = function(entry) {
    UserStudy.log("system\tdetect-pages");

    var doc = entry.browser.contentDocument;
    entry.detectedPages = true;
    
    var wizard = document.getElementById(entry.id).childNodes[0];
    try {
        if (PiggyBankExplorePane._detectPagesLinkLabelHeuristic(entry, wizard, doc)) {
            return;
        }
        if (PiggyBankExplorePane._detectPagesURLParamHeuristic(entry, wizard, doc)) {
            return;
        }
    } catch (e) {
        //
    }
    wizard.showSummaryPage(1);
}

PiggyBankExplorePane.resetPageLink = function(elmt) {
    var pane = elmt.parentNode.parentNode.parentNode.parentNode.parentNode.parentNode.parentNode;
    var entry = PiggyBankExplorePane._registry[pane.id];
    var deck = pane.childNodes[0].childNodes[1].childNodes[2].childNodes[1];
    
    entry.scraper.pagingData = null;
    deck.selectedIndex = 0;
}

PiggyBankExplorePane.browseSort = function(wizard) {
    UserStudy.log("user\tbrowse-sort\t");
    wizard.setDetectingItemsView();
    
    setTimeout(function() { PiggyBankExplorePane._detectItems(wizard); }, 10);
}

PiggyBankExplorePane._detectItems = function(wizard) {
    UserStudy.log("system\tdetecting-items\t");
    
    var pane = wizard.parentNode;
    var id = pane.id;
    var entry = PiggyBankExplorePane._registry[id];
    var doc = entry.browser.contentDocument;
    
    var xpaths = [];
    var xpathExists = [];
    
    var links = doc.links;
    for (var i = 0; i < links.length; i++) {
        var link = links[i];
        var xpath = PiggyBankExplorePane._generalizeXpath(link, doc);
/*        
        var firstChild = link.firstChild;
        if (firstChild != null) {
            xpath = xpath + "/" + (firstChild.nodeType == 1 ? firstChild.nodeName : "text()") + "[1]";
        }
*/        
        if (!xpathExists[xpath]) {
            xpathExists[xpath] = true;
            xpaths.push(xpath);
        }
    }
    
    var candidates = [];
    for (var i = 0; i < xpaths.length; i++) {
        var xpath = xpaths[i];
        var count = PiggyBankExplorePane._countXpath(doc, xpath);
        if (count > 3) {
            var itemXpath = xpath;
            while (true) {
                var xpath2 = itemXpath + "/..";
                var count2 = PiggyBankExplorePane._countXpath(doc, xpath2);
                if (count2 != count) {
                    break;
                } else {
                    itemXpath = xpath2;
                }
            }
            
            var area = 0;
            var left = Number.MAX_VALUE;
            var top = Number.MAX_VALUE;
            
            var iterator = doc.evaluate(itemXpath, doc, null, XPathResult.ANY_TYPE, null);
            var elmt = iterator.iterateNext();
            while (elmt) {
                var box = doc.getBoxObjectFor(elmt);
                area += box.width * box.height;
                left = Math.min(left, box.x);
                top = Math.min(top, box.y);
                elmt = iterator.iterateNext();
            }
            
            candidates.push({ itemXpath: itemXpath, area: area, left: left, top: top, count: count });
        }
    }
    
    /*
     *  Sort item Xpaths from largest area to smallest area
     */
    candidates.sort(function(a, b) {
        var c = b.area - a.area;
        if (c == 0) {
            c = b.count - a.count;
        }
        if (c == 0) {
            c = a.top - b.top;
        }
        if (c == 0) {
            c = a.left - b.left;
        }
        if (c == 0) {
            c = a.itemXpath.length - b.itemXpath.length;
        }
        return c;
    });
    
    /*
     *  Handle different coloring for odd/even rows using CSS class
     */
    if (candidates.length > 1) {
        var n = 1;
        while (n < candidates.length) {
            if (candidates[n].left != candidates[0].left ||
                candidates[n].top != candidates[0].top) {
                break;
            }
            n++;
        }
        
        if (n < candidates.length && Math.abs(candidates[0].count - candidates[n].count) < 2) {
            var xpath0Segments = candidates[0].itemXpath.split("/");
            var xpath1Segments = candidates[n].itemXpath.split("/");
         
            if (xpath0Segments.length == xpath1Segments.length) {
                var newSegments = [];
                for (var i = 0; i < xpath0Segments.length; i++) {
                    var segment0 = xpath0Segments[i];
                    var segment1 = xpath1Segments[i];
                    if (segment0 == segment1) {
                        newSegments.push(segment0);
                    } else {
                        var bracket0 = segment0.indexOf("[");
                        var bracket1 = segment1.indexOf("[");
                        
                        if (bracket0 < 0 || bracket1 < 0 ||
                            segment0.substr(0, bracket0) != segment1.substr(0, bracket1)) {
                            newSegments = null;
                            break;
                        } else {
                            var condition0 = segment0.substring(bracket0+1, segment0.length-1);
                            var condition1 = segment1.substring(bracket1+1, segment1.length-1);
                            var newSegment = 
                                segment0.substr(0, bracket0) + "[" + condition0 + " or " + condition1 + "]";
                                
                            newSegments.push(newSegment);
                        }
                    }
                }
                
                if (newSegments != null) {
                    candidates[0].itemXpath = newSegments.join("/");
                }
            }
        }
    }

    entry.scraper.itemXpath = candidates.length > 0 ? candidates[0].itemXpath : xpaths[0];
    entry.itemCount = PiggyBankExplorePane._countXpath(doc, entry.scraper.itemXpath);
    
    PiggyBankExplorePane._highlightItemXpath(id, doc, entry.scraper.itemXpath);
    wizard.setItemCount(entry.itemCount);
    
    UserStudy.log("system\tdetected-items\t" + entry.scraper.itemXpath);
    
    PiggyBankExplorePane._detectPages(entry);
        
    wizard.setDialogView();
}

PiggyBankExplorePane.locatePages = function(wizard) {
    UserStudy.log("user\tlocate-pages\t" + wizard.getSummaryPage());
    
    var pane = wizard.parentNode;
    var id = pane.id;
    var entry = PiggyBankExplorePane._registry[id];
    var doc = entry.browser.contentDocument;
    
    var done = function() {
        PiggyBankExplorePane._doneLocatingPages(id);
        
        wizard.setStopLocatingPagesHandler(null);
        wizard.setDialogView();
    };

    var listener = PiggyBankExplorePane._addCapturingListeners(
        doc, id, PiggyBankExplorePane._onCaptureLocatePageLinkNode, done);
    
    wizard.setLocatingPagesView();
    wizard.setStopLocatingPagesHandler(function() { listener.stop(); done(); } );
}

PiggyBankExplorePane.locateItems = function(wizard) {
    UserStudy.log("user\tlocate-items\t" + wizard.getSummaryPage());
    
    var pane = wizard.parentNode;
    var id = pane.id;
    var entry = PiggyBankExplorePane._registry[id];
    var doc = entry.browser.contentDocument;

    var done = function() {
        PiggyBankExplorePane._doneLocatingItems(entry);
        
        wizard.setStopLocatingItemsHandler(null);
        wizard.setDialogView();
    }
    
    entry.scraper.captureNode = null;
    entry.scraper.itemXpath = "";
    
    var listener = PiggyBankExplorePane._addCapturingListeners(
        doc, id, PiggyBankExplorePane._onCaptureSampleNode, done);
    
    wizard.setStopLocatingItemsHandler(function() { listener.stop(); done(); });
    
    wizard.locatingItemsHighlighter.unhighlight(entry.browser.contentDocument);
    wizard.locatingItemsHighlighter.drawWindow(entry.browser);
    wizard.setLocatingItemsView1();
    
    wizard.locatingItemsHighlighter.iframe.onmouseover = function() {
        wizard.setLocatingItemsView1();
    };
}

PiggyBankExplorePane.showItems = function(wizard) {
    UserStudy.log("user\tshow-items");
    
    var pane = wizard.parentNode;
    var entry = PiggyBankExplorePane._registry[pane.id];
    var doc = entry.browser.contentDocument;
    
    wizard.locatingItemsHighlighter.unhighlight(entry.browser.contentDocument);
    PiggyBankExplorePane._highlightItemXpath(entry.id, doc, entry.scraper.itemXpath);
}

PiggyBankExplorePane.showNextPageLink = function(wizard) {
    UserStudy.log("user\tshow-next-page-links");
    
    var pane = wizard.parentNode;
    var entry = PiggyBankExplorePane._registry[pane.id];
    var doc = entry.browser.contentDocument;
    
    var nodes = [];
    var results = entry.scraper.findNextPageLinks(doc, 1);
    for (var i = 0; i < results.length; i++) {
        nodes.push(results[i].elmt);
    }
    
    PiggyBankExplorePane._flashNodes(doc, nodes);
}

PiggyBankExplorePane.showDetectedPageLinks = function(wizard) {
    UserStudy.log("user\tshow-detected-page-links");
    
    var pane = wizard.parentNode;
    var entry = PiggyBankExplorePane._registry[pane.id];
    var doc = entry.browser.contentDocument;
    var nodes = entry.scraper.findDetectedPageLinks(doc, new PB_ScrapingUtilities());
    
    PiggyBankExplorePane._flashNodes(doc, nodes);
}

PiggyBankExplorePane._flashNodes = function(doc, nodes) {
    var outerDiv = doc.createElement("div");
    outerDiv.style.width = "0";
    outerDiv.style.height = "0";
    outerDiv.style.overflow = "visible";
    outerDiv.style.visibility = "hidden";
    
    var createImage = function(area) {
        var image = doc.createElement("img");
        image.setAttribute("src", "chrome://piggy-bank/skin/big-arrow-up.png");
        image.style.position = "absolute";
        image.style.width = "100px";
        image.style.height = "100px";
        image.style.left = ((area.left + area.right) / 2 - 50) + "px";
        image.style.top = (area.bottom + 5) + "px";
        
        return image;
    }
    
    var area = PiggyBankExplorePane._findArea(nodes[0]);
    area.centerX = (area.left + area.right) / 2;
    area.centerY = (area.top + area.bottom) / 2;
    
    for (var i = 1; i < nodes.length; i++) {
        var area2 = PiggyBankExplorePane._findArea(nodes[i]);
        area2.centerX = (area2.left + area2.right) / 2;
        area2.centerY = (area2.top + area2.bottom) / 2;
        
        if (Math.abs(area.centerX - area2.centerX) > 100 ||
            Math.abs(area.centerY - area2.centerY) > 100) {
            
            outerDiv.appendChild(createImage(area));
            area = area2;
        } else {
            area.left = Math.min(area.left, area2.left);
            area.right = Math.max(area.right, area2.right);
            area.top = Math.min(area.top, area2.top);
            area.bottom = Math.max(area.bottom, area2.bottom);
            area.centerX = (area.left + area.right) / 2;
            area.centerY = (area.top + area.bottom) / 2;
        }
    }
    outerDiv.appendChild(createImage(area));
    
    var count = 20;
    var f = function() {
        if ((count % 2) == 0) {
            outerDiv.style.visibility = "visible";
            if (count == 20) {
                outerDiv.firstChild.scrollIntoView(false);
            }
        } else {
            outerDiv.style.visibility = "hidden";
        }
        count--;
        
        if (count > 0) {
            doc.defaultView.setTimeout(arguments.callee, 100);
        } else {
            doc.body.removeChild(outerDiv);
        }
    }
    
    doc.body.appendChild(outerDiv);
    
    doc.defaultView.setTimeout(f, 100);
}

PiggyBankExplorePane._setScrapingUI = function(pane) {
    var wizard = pane.childNodes[0];
    wizard.setProcessingView();
}

PiggyBankExplorePane.startExploring = function(wizard) {
    UserStudy.log("user\tstart-exploring\t" + wizard.getSummaryPage() + "\t" + wizard.actionValue);
    
    var pane = wizard.parentNode;
    var entry = PiggyBankExplorePane._registry[pane.id];
    var browser = entry.browser;
    
    wizard.locatingItemsHighlighter.unhighlight(entry.browser.contentDocument);
    
    if (wizard.actionValue == "no-additional-pages") {
        entry.scraper.pagingData = { heuristic : "NoAdditionalPages" };
    }
    
/*    window.openDialog(
        "chrome://piggy-bank/content/align-dialog.xul", 
        "_blank", 
        "chrome,all,dialog=no,resizable,centerscreen,width=600,height=400", 
        { nodes : PiggyBankExplorePane._getNodesOnXPath(browser.contentDocument, entry.scraper.itemXpath) }
    );
*/

    PiggyBankExplorePane._setScrapingUI(pane);
    
    entry.scraper.canceled = false;
    
    var collector = PiggyBank._getCollector(entry.browser);
    setTimeout(
        function() {
            collector.autoCollectData(
                entry.scraper,
                function(profileID, tabBrowser, browser, results) {
                    entry.scraped = true;
                    entry.profileID = profileID;
                    entry.augment = results.augment;
                    entry.dynamicQuery = results.dynamicQuery;
                    
                    PiggyBankExplorePane._preparePane(pane);
                    PiggyBankExplorePane._fetchStart(entry, entry.dynamicQuery);
                    
                    PiggyBankExplorePane._saveAutoScraper(entry);
                }
            );
        },
        100
    );
}

PiggyBankExplorePane.stopExploring = function(wizard) {
    UserStudy.log("user\tstop-exploring");
    
    var pane = wizard.parentNode;
    var entry = PiggyBankExplorePane._registry[pane.id];
    
    entry.scraper.canceled = true;
    entry.browser.reload();
}

PiggyBankExplorePane._createCapturingMouseListener = function(
        doc, id, captureFunction, doneCaptureFunction) {
    return {
        _captured : false,
        _doc : doc,
        _id : id,
        _captureFunction : captureFunction,
        _doneCaptureFunction : doneCaptureFunction,
        
        handleEvent: function(aEvent) {
            var cancelEvent = function() {
                aEvent.stopPropagation();
                aEvent.preventBubble();
                aEvent.preventDefault();
                aEvent.cancelBubble = true;
            };
            
            var id = this._id;
            var doneCaptureFunction = this._doneCaptureFunction;
            var done = function() {
                doneCaptureFunction(id);
            }
            
            var elmt = aEvent.target;
            var tagName = elmt.tagName.toLowerCase();
            if (tagName != "html") {
                if (!this._captured && aEvent.type != "click") {
                    try {
                        this._captureFunction(elmt, this._id, aEvent);
                    } catch (e) {
                        printDebug(e);
                    }
                }
                
                if (aEvent.type == "mousedown") {
                    this._captured = true;
                } else if (aEvent.type == "click" ||
                          (aEvent.type == "mouseover" && this._captured)) {
                          
                    cancelEvent();
                    
                    PiggyBankExplorePane._removeCapturingListeners(this._doc, this);
                    setTimeout(done, 10);
                }
            }
            return false;
        },
        
        stop: function() {
            PiggyBankExplorePane._removeCapturingListeners(this._doc, this);
        }
    };
}

PiggyBankExplorePane._addCapturingListeners = function(doc, id, 
        captureFunction, doneCaptureFunction) {
    var listener = PiggyBankExplorePane._createCapturingMouseListener(
        doc, id, captureFunction, doneCaptureFunction);
        
    doc.addEventListener("mousedown", listener, true);
    doc.addEventListener("mousemove", listener, true);
    doc.addEventListener("click", listener, true);
    
    return listener;
}

PiggyBankExplorePane._removeCapturingListeners = function(doc, listener) {
    doc.removeEventListener("mousedown", listener, true);
    doc.removeEventListener("mousemove", listener, true);
    doc.removeEventListener("click", listener, true);
}

PiggyBankExplorePane._generalizeXpath = function(node, doc, keepIndices) {
    var namespace = doc.documentElement.namespaceURI;
    var prefix = namespace ? "x:" : "";
    
    var xpath = "";
    var node2 = node;
    while (node2 && node2 != doc) {
        var tag = node2.tagName.toLowerCase();
        var id = node2.id;
        var className = node2.className;
    
        var segment = prefix + tag;
        
        if (keepIndices) {
            if (tag == "tr") {
                var rowCount = node2.parentNode.rows.length;
                if (rowCount > 1 && rowCount < 5) {
                    segment += '[' + (node2.rowIndex+1) + ']';
                }
            } else if (tag == "td") {
                var cellCount = node2.parentNode.cells.length;
                if (cellCount > 1 && cellCount < 5) {
                    segment += '[' + (node2.cellIndex+1) + ']';
                }
            } else if (node2.nodeType == 1) {
                if (tag == "table" && tag == "div" && tag == "p") {
                    var siblings = node2.parentNode.getElementsByTagName(tag);
                    if (siblings.length < 5) {
                        for (var j = 0; j < siblings.length; j++) {
                            if (siblings[j] == node2) {
                                segment += '[' + (j+1) + ']';
                                break;
                            }
                        }
                    }
                }
            }
        }        
        
        if (className && className != "" && className.indexOf(" ") < 0) {
            segment += '[@class="' + className + '"]';
        }
        
        xpath = "/" + segment + xpath;
        
        node2 = node2.parentNode;
    }
    return xpath;
}

PiggyBankExplorePane._onCaptureLocatePageLinkNode = function(node, entryid, evt) {
    var doc = node.ownerDocument;
    var entry = PiggyBankExplorePane._registry[entryid];

    if (node.parentNode.id == "piggyBank-highlight-container") {
        node = PiggyBankExplorePane._hittestNonMarkerElement(doc.body, evt.pageX, evt.pageY);
    }
    if (entry.scraper.captureNode == node) {
        return;
    }
    
    while ((node) && node != doc.body) {
        if (node.nodeName == "A") {
            break;
        }
        node = node.parentNode;
    }
    
    entry.scraper.captureNode = node;
    
    var xpath = PiggyBankExplorePane._generalizeXpath(node, doc, true);
    if (xpath.length == 0) {
        return;
    }
    printDebug(xpath);
    
    entry.scraper.pagingData = { 
        xpath:      xpath,
        heuristic:  "NextPage"
    };
    
    /*
     *  Count unique links
     */
    var iterator = doc.evaluate(xpath, doc, null, XPathResult.ANY_TYPE, null);
    var elmt = iterator.iterateNext();
    var links = [];
    while (elmt) {
        if (elmt.textContent.search(/\d+/) >= 0) {
            links.push(elmt.href);
        }
        elmt = iterator.iterateNext();
    }
    
    links = links.sort();
    var count = 1;
    var l = links[0];
    for (var x = 1; x < links.length; x++) {
        if (l != links[x]) {
            l = links[x];
            count++;
        }
    }
    
    entry.pageLinkCount = count + 1; // count current page, too
}

PiggyBankExplorePane._doneLocatingPages = function(entryid) {
    var entry = PiggyBankExplorePane._registry[entryid];
    entry.scraper.captureNode = null;
    
    var pane = document.getElementById(entry.id);
    var wizard = pane.childNodes[0];
    
    wizard.showSummaryPage(2);
    wizard.setDialogView();
    
    UserStudy.log("user\tcaptured-page-link\t" + entry.scraper.pagingData.xpath);
}

PiggyBankExplorePane._onCaptureSampleNode = function(node, entryid, evt) {
    var doc = node.ownerDocument;
    var entry = PiggyBankExplorePane._registry[entryid];

    if (node.parentNode.id == "piggyBank-highlight-container") {
        node = PiggyBankExplorePane._hittestNonMarkerElement(doc.body, evt.pageX, evt.pageY);
    }
    if (entry.scraper.captureNode == node) {
        return;
    }
    entry.scraper.captureNode = node;
    
    var xpath = PiggyBankExplorePane._generalizeXpath(node, doc);
    

    /*
     *  Check if Xpath makes sense
     */
    var count = PiggyBankExplorePane._countXpath(doc, xpath);
    if (count < 5 || count > 100) {
        return;
    }
    
    /*
     *  Expand selection from fields to items
     */
    
    var itemXpath = xpath;
    entry.itemCount = count;
    while (true) {
        var xpath2 = itemXpath + "/..";
        var count2 = PiggyBankExplorePane._countXpath(doc, xpath2);
        if (count2 != count) {
            break;
        } else {
            itemXpath = xpath2;
        }
    }
    
    if (itemXpath == entry.scraper.itemXpath) {
        return;
    }
    entry.scraper.itemXpath = itemXpath;
    
    /*
     *  Re-highlight new xpath
     */
    PiggyBankExplorePane._highlightItemXpath(entryid, doc, entry.scraper.itemXpath);
    
    var pane = document.getElementById(entryid);
    var wizard = pane.childNodes[0];
    wizard.setLocatingItemsView2();
    
    UserStudy.log("user\tcapturing-items\t" + entry.scraper.itemXpath);
}

PiggyBankExplorePane._doneLocatingItems = function(entry) {
    entry.scraper.captureNode = null;
    
    var pane = document.getElementById(entry.id);
    var wizard = pane.childNodes[0];
    
    wizard.setItemCount(entry.itemCount);
    if (!entry.detectedPages) {
        PiggyBankExplorePane._detectPages(entry);
    } else if (entry.pageCount) {
        wizard.setTotalItemCountText(
            (entry.itemCount * (entry.pageCount - 1)) + " ~ " + (entry.itemCount * entry.pageCount)
        );
    }
    
    UserStudy.log("user\tcaptured-items\t" + entry.scraper.itemXpath);
}

PiggyBankExplorePane._hittestNonMarkerElement = function(elmt, x, y) {
    var childNodes = elmt.childNodes;
    for (var i = 0; i < childNodes.length; i++) {
        var childNode = childNodes[i];
        if (childNode.nodeType != 1 || childNode.id == "piggyBank-highlight-container") {
            continue;
        }
        
        if (childNode.offsetWidth == 0 && childNode.offsetHeight == 0) {
            /*
             *  Sometimes SPAN elements have zero width and height but
             *  they have children like DIVs that cover non-zero areas.
             */
            var hitNode = PiggyBankExplorePane._hittestNonMarkerElement(childNode, x, y);
            if (hitNode != childNode) {
                return hitNode;
            }
        } else {
            var top = 0;
            var left = 0;
            
            var node = childNode;
            while (node) {
                top += node.offsetTop;
                left += node.offsetLeft;
                node = node.offsetParent;
            }
            
            if (left <= x && top <= y && (x - left) < childNode.offsetWidth && (y - top) < childNode.offsetHeight) {
                return PiggyBankExplorePane._hittestNonMarkerElement(childNode, x, y);
            }
        }
    }
    return elmt;
}

PiggyBankExplorePane._findArea = function(node) {
    if (node.nodeType != 1) {
        return PiggyBankExplorePane._findArea(node.parentNode);
    } else if (node.offsetWidth == 0 || node.offsetHeight == 0) {
        var left = node.ownerDocument.width;
        var right = 0;
        var top = node.ownerDocument.height;
        var bottom = 0;
        
        for (var i = 0; i < node.childNodes.length; i++) {
            var childNode = node.childNodes[i];
            if (childNode.nodeType == 1) {
                var area = PiggyBankExplorePane._findArea(childNode);
                
                left = Math.min(left, area.left);
                top = Math.min(top, area.top);
                right = Math.max(right, area.right);
                bottom = Math.max(bottom, area.bottom);
            }
        }
        
        return { left: left, top: top, right: right, bottom: bottom };
    } else {
        var top = 0;
        var left = 0;
        var width = node.offsetWidth;
        var height = node.offsetHeight;
        
        var node2 = node;
        while (node2) {
            top += node2.offsetTop;
            left += node2.offsetLeft;
            node2 = node2.offsetParent;
        }
        
        return { left: left, top: top, right: left + width, bottom: top + height };
    }
}

PiggyBankExplorePane._highlightItemXpath = function(entryid, doc, xpath) {
    var nodes = PiggyBankExplorePane._getNodesOnXPath(doc, xpath);
    var wizard = document.getElementById(entryid).childNodes[0];
    wizard.locatingItemsHighlighter.highlight(doc, nodes);
}

PiggyBankExplorePane._getNodesOnXPath = function(doc, xpath) {
    var nodes = [];
    var iterator = doc.evaluate(xpath, doc, null, XPathResult.ANY_TYPE,null);
    var node = iterator.iterateNext();
    while (node) {
        nodes.push(node);
        node = iterator.iterateNext();
    }
    return nodes;
}

PiggyBankExplorePane._countXpath = function(doc, xpath) {
    return doc.evaluate("count(" + xpath + ")", doc, null, XPathResult.ANY_TYPE, null).numberValue;
}
