const PB_TOKEN_TEXT = 0;
const PB_TOKEN_WHITESPACE = 1;
const PB_TOKEN_NUMBER = 2;
const PB_TOKEN_DATE = 3;

function PBToken(text, type) {
    this.text = text;
    this.type = type;
}

PBToken.splitTokenize = function(s, match, parse, keepWhitespace) {
    var index = s.indexOf(match);
        
    var tokens = PBToken.tokenize(s.substr(0, index), "", keepWhitespace);
    tokens.push(parse(match));
    
    return tokens.concat(PBToken.tokenize(s.substr(index + match.length), "", keepWhitespace));
}

PBToken._monthPrefixes = [ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" ];
PBToken._parseDate1 = function(s) {
    var token = new PBToken(s, PB_TOKEN_DATE);
    
    var monthPrefix = s.substr(0, 3);
    var month = 1;
    while (month <= PBToken._monthPrefixes.length && PBToken._monthPrefixes[month-1] != monthPrefix) {
        month++;
    }
    s = s.substr(s.search(/\d/));
    
    var dayEnd = s.search(/\D/);
    var day = parseInt(s.substr(0, dayEnd));
    
    s = s.substr(dayEnd);
    s = s.substr(s.search(/\d/));
    
    var yearEnd = s.search(/\D/);
    if (yearEnd < 0) {
        yearEnd = s.length;
    }
    var year = s.substr(0, yearEnd);
    if (year.length == 2) {
        if (year.substr(0,1) == "0") {
            year = "20" + year;
        } else {
            year = "19" + year;
        }
    }
    
    s = s.substr(yearEnd);
    
    var hour = 0;
    var minute = "00";
    var second = "00";
    if (s.length > 0) {
        s = s.substr(s.search(/\d/));
        
        var hourEnd = s.indexOf(":");
        hour = parseInt(s.substr(0, hourEnd));
        
        s = s.substr(hourEnd + 1);
        minute = s.substr(0, 2);
        
        s = s.substr(2);
        if (s.substr(0, 1) == ":") {
            second = s.substr(1, 2);
            s = s.substr(3);
        }
        
        s = s.substr(s.search(/\w/));
        if (s.substr(0,1).toLowerCase() == "p" && hour < 12) {
            hour += 12;
        }
    }
    
    token.value = 
        year + "-" + 
        (month < 10 ? "0" + month : month) + "-" + 
        (day < 10 ? "0" + day : day) + "T" +
        (hour < 10 ? "0" + hour : hour) + ":" +
        minute + ":" +
        second + "Z";

    return token;
}

PBToken._parseNumber = function(s) {
    var token = new PBToken(s, PB_TOKEN_NUMBER);
    token.value = parseFloat(s.split(",").join(""));
    
    return token;
}

PBToken.tokenize = function(s, nodeName, keepWhitespace) {
    if (nodeName == "@src") {
        return new PBToken(s, 0);
    }
    
    var tokens = [];
    
    var trimLeft = function() {
        var i = s.search(/\S/);
        if (i > 0) {
            if (keepWhitespace) {
                tokens.push(new PBToken(s.substr(0, i), PB_TOKEN_WHITESPACE));
            }
            s = s.substr(i);
        } else if (i < 0) {
            if (keepWhitespace) {
                tokens.push(new PBToken(s, PB_TOKEN_WHITESPACE));
            }
            return true;
        }
        return false;
    }
    
    if (trimLeft()) {
        return tokens;
    }
    
    var dates = s.match(/(Jan(uary)?|Feb(ruary)?|Mar(ch)?|Apr(il)?|May|Jun(e)?|Jul(y)?|Aug(ust)?|Sep(t|tempber)?|Oct(ober)?|Nov(ember)?|Dec(ember)?)\s*\d?\d(\,\s*|\s+)(\d\d)\d\d(\s+\d?\d:\d\d(:\d\d)?\s*(am|AM|a|A|pm|PM|p|P)?)?/);
    if (dates && dates.length > 0) {
        return tokens.concat(PBToken.splitTokenize(s, dates[0], PBToken._parseDate1, keepWhitespace));
    }
    
    var floats = s.match(/[-\+]?\d+(\,\d\d\d)*(\.\d+)?/);
    if (floats && floats.length > 0) {
        return tokens.concat(PBToken.splitTokenize(s, floats[0], PBToken._parseNumber, keepWhitespace));
    }
    
    while (s.length > 0) {
        var i = s.search(/\W/);
        if (i == 0) {
            tokens.push(new PBToken(s.substr(0, 1), PB_TOKEN_TEXT));
            s = s.substr(1);
        } else if (i > 0) {
            tokens.push(new PBToken(s.substr(0, i), PB_TOKEN_TEXT));
            s = s.substr(i);
        } else {
            tokens.push(new PBToken(s, PB_TOKEN_TEXT));
            break;
        }
        
        if (trimLeft()) {
            break;
        }
    }
    return tokens;
}
