/*----------------------------------------------------------------------
 * The nsClassLoader Module
 *----------------------------------------------------------------------
 */

var ClassLoaderModule = {
    myCID : Components.ID("{5eef07e0-0ab6-11da-8cd6-0800200c9a66}"),
    myProgID : "@simile.mit.edu/class-loader;1",

    registerSelf : function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(
            this.myCID,
            "Custom Java Class Loader component",
            this.myProgID,
            fileSpec,
            location,
            type
        );
    },

    unregisterSelf : function(compMgr, fileSpec, location) {
        compMgr = compMgr.QueryInterface(nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(this.myCID, fileSpec);
    },

    getClassObject : function(compMgr, cid, iid) {
        if (!cid.equals(this.myCID)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        if (!iid.equals(Components.interfaces.nsIFactory)) {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
        return this.myFactory;
    },

    canUnload : function(compMgr) {
        ClassLoader_singleton = null;        
        return true;
    },

    myFactory : {
        createInstance : function(outer, iid) {
            if (outer != null) {
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            }

            if (!ClassLoader_singleton) {
                ClassLoader_singleton = new ClassLoader();
            }

            return ClassLoader_singleton.QueryInterface(iid);
        }
    }
}

function NSGetModule(compMgr, fileSpec) {
    return ClassLoaderModule;
}

/*----------------------------------------------------------------------
 * The ClassLoader Component
 *----------------------------------------------------------------------
 */

var ClassLoader_singleton = null;

function ClassLoader() {
    this.wrappedJSObject = this;
    this.initialized = false;
}

ClassLoader.prototype = {

    QueryInterface : function(iid) {
        if (!iid.equals(Components.interfaces.nsIClassLoader) &&
            !iid.equals(Components.interfaces.nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    },

    _initialize : function(java, classLoaderJarPath) {
        
        this._trace("> initialize");

        if (this._initialized) {
            this._trace("--[already initialized]--")
            this._trace("< initialize")
            return;
        }

        // we create a classloader to class we made to allow permissions
        // to load classes from a particular URL (our local jars, that is)
        var classLoaderURL = new java.net.URL(classLoaderJarPath);
        var bootstrapClassLoader = java.net.URLClassLoader.newInstance([classLoaderURL]);
        var policyClass = java.lang.Class.forName(
            "edu.mit.simile.piggyBank.classLoader.URLSetPolicy",
            true,
            bootstrapClassLoader);
        
        // then we inject this class into the security policy of the JVM security manager
        // to have permission to load our own classloader from the classloading jar.
        this._policy = policyClass.newInstance();
        this._policy.setOuterPolicy(java.security.Policy.getPolicy());
        java.security.Policy.setPolicy(this._policy);
        this._policy.addPermission(new java.security.AllPermission());
        this._policy.addURL(classLoaderURL);

        // now that we gave ourselves permission, we can load our own classloader
        // thru the regular system classloader
        var classLoaderClassLoader = java.net.URLClassLoader.newInstance([classLoaderURL]);
        this._classLoaderClass = new ClassLoader_Packages(java,classLoaderClassLoader).
            getClass("edu.mit.simile.piggyBank.classLoader.TracingClassLoader");

        // with a reference to our own classloader class we are ready to rock!
        this._initialized = true;
        
        this._trace("< initialize");
    },

    getPackages : function(java, classLoaderJarPath, jars, trace) {
        this._trace("> getPackages");

        // obtain a reference to our own classloader class
        this._initialize(java, classLoaderJarPath);
        
        // add all the requested jars to the security policy, to allow loading classes from them
        var jarURLs = [ new java.net.URL(classLoaderJarPath) ];
        this._policy.addURL(jarURLs[0]);
        for (var i = 0; i < jars.length; i++) {
            var jarURL = new java.net.URL(jars[i]);
            this._policy.addURL(jarURL);
            jarURLs.unshift(jarURL);
        }

        if (!trace) trace = false; // make sure trace is a boolean (could be undefined or null)
        
        // instantiate the new classloader with the given set of jars 
        // and the appropriate security policy and set it as the 
        // context classloader of this thread
        var classLoader = this._classLoaderClass.m("newInstance")(trace);
        for (var i = 0; i < jarURLs.length; i++) {
            classLoader.add(jarURLs[i]);
        }
        java.lang.Thread.currentThread().setContextClassLoader(classLoader);

        this._trace("< getPackages");

        // return the classloader wrapped with some javascript voodoo to perform
        // class loading, instantiating and method calling.
        return new ClassLoader_Packages(java, classLoader);
    },
        
    _trace : function (msg) {
        Components.classes["@simile.mit.edu/logger;1"]
            .getService(Components.interfaces.nsILogger)
                .trace("nsClassloader",msg);
    }

};

function ClassLoader_Packages(java, classLoader) {

    var _classLoader = classLoader;
    var _thePackages = java.lang.Class.forName(
        "edu.mit.simile.piggyBank.classLoader.Packages",
        true,
        classLoader
    ).newInstance();

    this.getClass = function(className) {
        var classWrapper = _thePackages.getClass(className);
        if (classWrapper) {
            return {
                n : function() {
                    return classWrapper.callConstructor(ClassLoader_argumentsToArray(arguments));
                },
                f : function(fieldName) {
                    return classWrapper.getField(fieldName);
                },
                m : function(methodName) {
                    return function() {
                        return classWrapper.callMethod(methodName, ClassLoader_argumentsToArray(arguments));
                    };
                }
            };
        } else {
            return null;
        }
    };
    
    this.setTracing = function(enable) {
        _classLoader.setTracing((enable) ? true : false);
    };
}

function ClassLoader_argumentsToArray(args) {
    var a = [];
    for (var i = 0; i < args.length; i++) {
        a[i] = args[i];
    }
    return a;
}
