/*----------------------------------------------------------------------
 * The nsExplorer Module
 *----------------------------------------------------------------------
 */

function printToJSConsole(msg) {
    Components.classes["@mozilla.org/consoleservice;1"]
        .getService(Components.interfaces.nsIConsoleService)
            .logStringMessage(msg);
}

function printObjectToJSConsole(o) {
    var a = [];
    for (x in o) {
        a.push(x + " = " + o[x]);
    }
    printToJSConsole(a.join("\n"));
}

function inspect(o) {
    var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
        .getService(Components.interfaces.nsIWindowWatcher);
        
    ww.activeWindow.openDialog("chrome://inspector/content/object.xul", "_blank", "chrome,all,dialog=no", o);
}

var ExplorerModule = {
    _componentID : Components.ID("{03C6EEAC-8714-4d8d-A77F-35D0BE617D94}"),
    _contractID : "@simile.mit.edu/explorer;1",
    _name : "Explorer component",

    registerSelf : function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(
            this._componentID,
            this._name,
            this._contractID,
            fileSpec,
            location,
            type
        );
        
        /*
         *  Adapted from nsSidebar.js in Firefox 1.5 in order
         *  to make the "data" variable accessible to Web pages'
         *  Javascript, much as as the "sidebar" variable is
         *  accessible.
         */
        var catman = 
            Components.classes["@mozilla.org/categorymanager;1"].
                getService(Components.interfaces.nsICategoryManager);
                                
        catman.addCategoryEntry(
            "JavaScript global property",
            "explorer",
            this._contractID,
            true,
            true
        );
    },

    unregisterSelf : function(compMgr, fileSpec, location) {
        compMgr = compMgr.QueryInterface(nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(this._componentID, fileSpec);
    },

    getClassObject : function(compMgr, cid, iid) {
        if (!cid.equals(this._componentID)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        if (!iid.equals(Components.interfaces.nsIFactory)) {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
        return this._factory;
    },

    canUnload : function(compMgr) {
        return true;
    },

    _factory : {
        createInstance : function(outer, iid) {
            if (outer != null) {
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            }
            
            return new Explorer().QueryInterface(iid);
        }
    }
}

function NSGetModule(compMgr, fileSpec) {
    return ExplorerModule;
}

/*----------------------------------------------------------------------
 * The Explorer Component
 *----------------------------------------------------------------------
 */

var count = 0;

function Explorer() {
    this.wrappedJSObject = this;
    
    this._initialized = false;
    this.itemCount = 0;
    this._key = Math.random();
}

Explorer.prototype.QueryInterface = function(iid) {
    if (!iid.equals(Components.interfaces.nsIExplorer) &&
        !iid.equals(Components.interfaces.nsISupports) &&
        !iid.equals(Components.interfaces.nsIClassInfo)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
};

    /*------------------------------------------------------------------
     * nsIClassInfo
     *------------------------------------------------------------------
     */

Explorer.prototype.flags = Components.interfaces.nsIClassInfo.DOM_OBJECT;
Explorer.prototype.classDescription = "Explorer";
Explorer.prototype.getInterfaces = function(count) {
    var interfaceList = [
        Components.interfaces.nsIExplorer, 
        Components.interfaces.nsIClassInfo
    ];
    count.value = interfaceList.length;
    return interfaceList;
};
Explorer.prototype.getHelperForLanguage = function(count) {
    return null;
};

    /*------------------------------------------------------------------
     * nsIExplorer
     *------------------------------------------------------------------
     */
Explorer.prototype.hoverSmartTag = function(propertyURIs) {
    var a = this._findWindow();
    var window = a[3];
    var browser = a[1];
    var id = browser.getAttribute("piggyBank-key");
    
    window.PiggyBankExplorePane.hoverSmartTag(id, propertyURIs);
};

Explorer.prototype.selectSmartTag = function(propertyURIs) {
    var a = this._findWindow();
    var window = a[3];
    var browser = a[1];
    var id = browser.getAttribute("piggyBank-key");
    
    window.PiggyBankExplorePane.selectSmartTag(id, propertyURIs);
};

    /*------------------------------------------------------------------
     * Internal implementation
     *------------------------------------------------------------------
     */
     
Explorer.prototype._initialize = function() {
    if (!this._initialized) {
        printToJSConsole(this._findWindow().document.location);
        this._initialized = true;
    }
};

Explorer.prototype._findWindow = function() {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
        .getService(Components.interfaces.nsIWindowMediator);
    var e = wm.getEnumerator("navigator:browser");
    while (e.hasMoreElements()) {
        var w = e.getNext();
        var tabBrowser = w.document.getElementById("content");
        if (tabBrowser) {
            var browsers = tabBrowser.browsers;
            for (var i = 0; i < browsers.length; i++) {
                var browser = browsers[i];
                try {
                    var window = browser.contentWindow.top;
                    var explorer = window.explorer;
                    if (explorer.wrappedJSObject._key == this._key) {
                        return [ window, browser, tabBrowser, w ];
                    }
                } catch (e) {
                    // silent
                }
            }
        }
    }
    return null;
};

Explorer.prototype.doIt = function() {
    this._initialize();
};