/*----------------------------------------------------------------------
 * The PiggyBank Module
 *----------------------------------------------------------------------
 */

var PiggyBankModule = {
    myCID : Components.ID("{7a97df30-0ab6-11da-8cd6-0800200c9a66}"),
    myProgID : "@simile.mit.edu/piggy-bank;1",

    registerSelf : function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(
            this.myCID,
            "Piggy Bank component",
            this.myProgID,
            fileSpec,
            location,
            type
        );
    },

    unregisterSelf : function(compMgr, fileSpec, location) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(this.myCID, fileSpec);
        compMgr.unregisterFactoryLocation(this.myHandlerCID, fileSpec);
    },

    getClassObject : function(compMgr, cid, iid) {

        if (cid.equals(this.myCID)) {
            return this.myFactory;
        } else if (!iid.equals(Components.interfaces.nsIFactory)) {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }

        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    canUnload : function(compMgr) {
        PiggyBank_singleton._piggyBankInstance.shutdown();
        PiggyBank_singleton = null;
        return true;
    },

    myFactory : {
        createInstance : function(outer, iid) {
            if (outer != null) {
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            }

            if (!PiggyBank_singleton) {
                PiggyBank_singleton = new PiggyBank();
            }

            return PiggyBank_singleton.QueryInterface(iid);
        }
    }
}

function NSGetModule(compMgr, fileSpec) {
    return PiggyBankModule;
}

/*----------------------------------------------------------------------
 * The PiggyBank Component
 *----------------------------------------------------------------------
 */

var PiggyBank_singleton = null;

function PiggyBank() {
    this.version = "2.1.2"; // NOTE: this gets expanded in the build process!
    this.wrappedJSObject = this;
    this.initialized = false;
    this.loading = false;
    this.attempted = false;
    this.error = null;
    this._piggyBankInstance = null;
    this._packages = null;
}

PiggyBank.prototype = {

    QueryInterface : function(iid) {
        if (!iid.equals(Components.interfaces.nsIPiggyBank) &&
            !iid.equals(Components.interfaces.nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    },

    fillTasks : function(listbox) {
        var tasks = [
            "Load Java code",
            "Start up Piggy Bank server",
            "Populate bank accounts",
            "Migrate data from previous installations",
            "Load ontologies",
            "Load configurations",
            "Load facades"
        ];

        for (var i = 0; i < tasks.length; i++) {
            var item = listbox.appendItem(tasks[i]);
            item.setAttribute("type", "checkbox");
        }

        return tasks.length;
    },

    initialize : function (
        java,
        previousVersion,
        port,
        extensionPath,
        profilePath,
        idForGeneratingTagURIs,
        googleKey,
        traceClassloader,
        traceJava,
        piggybankClasses,
        longwellClasses,
        progress
    ) {
        this._trace("> initialize");

        this.attempted = true;

        if (this.initialized) {
            this._trace("--[already initialized]--")
            this._trace("< initialize")
            progress.done();
            return;
        }

        var libPath = extensionPath + "components/lib/";
        var rdfPath = extensionPath + "rdf/";
        var classloaderJar = extensionPath + "components/lib/class-loader.jar";
        var commonJarFilenames =  ['informa-0.6.0.jar','jdom-1.0.jar','jtidy-04aug2000r7-dev.jar','antlr.jar','batik-awt-util-1.5.1.jar','commons-codec-1.3.jar','commons-collections-3.1.jar','commons-lang-2.1.jar','commons-logging-1.0.4.jar','concurrent-1.3.4.jar','fresnel.jar','isaviz-fsl.jar','javax.servlet-5.1.4.jar','jdom-1.0.jar','jetty-5.1.4.jar','log4j-1.2.11.jar','lucene-1.4.3.jar','openrdf-model-sesame-2a3.jar','openrdf-util-sesame-2a3.jar','picto-1.0.0.jar','retroweaver-rt-1.2.2.jar','rio-2a3.jar','sesame-2a3.jar','sparql-core-sesame-2a3.jar','sparql-sesame-2a3.jar','velocity-r398041.jar','vocabularies-fresnel.jar','whirlycache-0.7.1.jar']; // NOTE: this gets expanded in the build process!
        var jars = [];

        if (longwellClasses && longwellClasses != "") {
            if (longwellClasses.substring(0,8) != "file:///") longwellClasses = "file:///" + longwellClasses;
            if (longwellClasses.indexOf(-1) != "/") longwellClasses += "/";
            jars.unshift(longwellClasses);
        } else {
            commonJarFilenames.unshift("longwell.jar");
        }

        if (piggybankClasses && piggybankClasses != "") {
            if (piggybankClasses.substring(0,8) != "file:///") piggybankClasses = "file:///" + piggybankClasses;
            if (piggybankClasses.indexOf(-1) != "/") piggybankClasses += "/";
            jars.unshift(piggybankClasses);
        } else {
            commonJarFilenames.unshift("piggy-bank.jar");
        }

        for (var i = 0; i < commonJarFilenames.length; i++) {
            jars.unshift(libPath + commonJarFilenames[i]);
        }

        this._trace("> Start Java");
        var classLoader = Components.classes["@simile.mit.edu/class-loader;1"].getService(Components.interfaces.nsIClassLoader);
        this._packages = classLoader.wrappedJSObject.getPackages(java, classloaderJar, jars, traceClassloader);
        progress.taskCompleted();
        this._trace("< Start Java");

        if (!traceJava) traceJava = false; // make sure it's a boolean

        var piggyBankClass = this._packages.getClass("edu.mit.simile.piggyBank.PiggyBank");
        if (piggyBankClass) {
            try {
                this._trace("> Start Piggy Bank");
                piggyBankClass.m("startup")(
                    profilePath + "piggy-bank/", // user data dir
                    extensionPath, // extension installation dir
                    port, // port
                    idForGeneratingTagURIs,
                    googleKey,
                    traceJava
                );
                this._piggyBankInstance = piggyBankClass.n();
                progress.taskCompleted();
                this._trace("< Start Piggy Bank");

                this._trace("> Populate Accounts");
                this._populateAccounts();
                progress.taskCompleted();
                this._trace("< Populate Accounts");

                this._trace("> Migrate old Data (" + previousVersion + ")");
                // sesame does this automatically, so we are basically faking it here, eheh
                progress.taskCompleted();
                this._trace("< Migrate old Data");

                this._trace("> Initialize Data");
                if (this.version != previousVersion) {
                    this._initializeData(java, rdfPath, progress);
                } else {
                    // if not needed, fake it
                    progress.taskCompleted();
                    progress.taskCompleted();
                    progress.taskCompleted();
                }
                this._trace("< Initialize Data");

                this.initialized = true;

                progress.done();
            } catch (e) {
                this.fail(e);
                progress.fail(this.error);
            }
        } else {
            this.error = "Could not find the main PiggyBank class. There could be something wrong in the classpath: \n  " + jars.join('\n  ');
            progress.fail(this.error);
        }

        this._trace("< initialize");
    },

    getInstance : function() {
        this._trace("getJavaInstance");
        return this._piggyBankInstance;
    },

    dispose : function() {
        this._trace("dispose");
        this._piggyBankInstance.shutdown();
        this.initialized = false;
        this.loading = false;
        this.attempted = false;
        this.error = null;
        this._piggyBankInstance = null;
        this._packages = null;
    },

    fail : function(e) {
        if (e.getMessage) {
            this.error = e + ": " + e.getMessage() + "\n";
            while (e.getCause() != null) {
                e = e.getCause();
                this.error += "caused by " + e + ": " + e.getMessage() + "\n";
            }
        } else {
            this.error = e;
        }
    },

    setClassloaderTracing : function(enable) {
        if (this._packages) {
            this._packages.setTracing(enable);
        }
    },

    setJavaTracing : function(enable) {
        if (this._piggybankInstance) {
            this._piggyBankInstance.setTracing(enable);
        }
    },

    setGoogleKey : function(key) {
        if (this._piggybankInstance) {
            this._piggyBankInstance.setGoogleKey(key);
        }
    },

    showWizard : function(previous_version) {
        return ((previous_version == "") || (previous_version.substring(0,4) == "2.0."));
    },

    _populateAccounts : function() {
        this._trace("> _populateAccounts");

        var localStore =
            Components.classes["@mozilla.org/rdf/datasource;1?name=local-store"]
                .getService(Components.interfaces.nsIRDFDataSource);
        var rdfService =
            Components.classes["@mozilla.org/rdf/rdf-service;1"]
                .getService(Components.interfaces.nsIRDFService);

        var pbNamespace = "http://simile.mit.edu/2005/04/piggy-bank#";
        var typeProperty = rdfService.GetResource("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
        var accountTypeResource = rdfService.GetResource(pbNamespace + "Account");
        var bankAddressProperty = rdfService.GetResource(pbNamespace + "bankAddress");
        var accountIDProperty = rdfService.GetResource(pbNamespace + "accountID");
        var passwordHashProperty = rdfService.GetResource(pbNamespace + "passwordHash");

        var accounts = localStore.GetSources(typeProperty, accountTypeResource, true);
        while (accounts.hasMoreElements()) {
            var account = accounts.getNext();
            var accountURI = account.Value;

            var bankAddress = localStore.GetTarget(account, bankAddressProperty, true).
                QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
            var accountID = localStore.GetTarget(account, accountIDProperty, true).
                QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
            var passwordHash = localStore.GetTarget(account, passwordHashProperty, true).
                QueryInterface(Components.interfaces.nsIRDFLiteral).Value;

            this._piggyBankInstance.addAccount(accountID, passwordHash, bankAddress);
        }

        this._trace("< _populateAccounts");
    },

    _initializeData : function(java, rdfPath, progress) {
        this._trace("> _initializeData")

        var defaultProfile = this._piggyBankInstance.getPiggyBankServer().getDefaultProfile();
        defaultProfile.addData(new java.io.File(new java.net.URI(rdfPath + "ontologies/")));
        progress.taskCompleted();
        defaultProfile.addData(new java.io.File(new java.net.URI(rdfPath + "configurations/")));
        progress.taskCompleted();
        defaultProfile.addData(new java.io.File(new java.net.URI(rdfPath + "facades/")));
        progress.taskCompleted();

        this._trace("< _initializeData");
    },

    _trace : function (msg) {
        Components.classes["@simile.mit.edu/logger;1"]
            .getService(Components.interfaces.nsILogger)
                .trace("nsPiggyBank",msg);
    }
}
